@@ -25,4 +25,10 @@ void ErrPointer(PyMOLGlobals * G, const char *file, int line);
 int ErrMessage(PyMOLGlobals * G, const char *where, const char *what);
 
 #define ErrChkPtr(G,p) {if(!p) ErrPointer(G,__FILE__,__LINE__);}
+
+#define CHECKOK(ok, var) ok &= var ? true : false;
+
+#define th_raise(x) goto th_except ## x
+#define th_assert(x, c) {if(!(c)) th_raise(x);}
+
 #endif
