@@ -624,200 +624,6 @@ int MatchAlign(CMatch * I, float gap_penalty, float ext_penalty,
   return (ok);
 }
 
-#if 0
-int MatchAlign(CMatch * I, float gap_penalty, float ext_penalty,
-               int max_gap, int max_skip, int quiet)
-{
-  PyMOLGlobals *G = I->G;
-  unsigned int dim[2];
-  register int a, b, f, g;
-  register int nf, ng;
-  register int sf, sg;
-  register float **score;
-  register int na = I->na, nb = I->nb;
-  register int2 **point;
-  float mxv;
-  int mxa, mxb;
-  float tst = 0.0;
-  register int gap = 0;
-  int *p;
-  int cnt;
-  int ok = true;
-  const float MIN_SCORE = 0.0F;
-  nf = na + 1;
-  ng = nb + 1;
-
-  if(!quiet) {
-    PRINTFB(G, FB_Match, FB_Actions)
-      " MatchAlign: aligning residues (%d vs %d)...\n", na, nb ENDFB(G);
-  }
-
-  dim[0] = nf;
-  dim[1] = ng;
-  VLAFreeP(I->pair);
-  score = (float **) UtilArrayCalloc(dim, 2, sizeof(float));
-  point = (int2 **) UtilArrayCalloc(dim, 2, sizeof(int2));
-  if(score && point) {
-
-    /* initialize the scoring matrix */
-    for(f = 0; f < nf; f++) {
-      for(g = 0; g < ng; g++) {
-        score[f][g] = MIN_SCORE;
-        point[f][g][0] = -1;
-        point[f][g][1] = -1;
-      }
-    }
-    /* now start walking backwards up the alignment */
-
-    {
-      int second_pass = false;
-      for(b = nb - 1; b >= 0; b--) {
-        for(a = na - 1; a >= 0; a--) {
-
-          /* find the maximum scoring cell accessible from this position, 
-           * while taking gap penalties into account */
-
-          mxv = MIN_SCORE;
-          mxa = -1;
-          mxb = -1;
-
-          /* search for asymmetric insertions and deletions */
-          f = a + 1;
-          if((max_gap >= 0) && (second_pass)) {
-            sf = a + 2 + max_gap;
-            sg = b + 2 + max_gap;
-            if(sg > ng)
-              sg = ng;
-            if(sf > nf)
-              sf = nf;
-          } else {
-            sg = ng;
-            sf = nf;
-          }
-          for(g = b + 1; g < sg; g++) {
-            tst = score[f][g];
-            if(!((f == na) || (g == nb))) {
-              gap = g - (b + 1);
-              if(gap)
-                tst += gap_penalty + ext_penalty * (gap - 1);
-            }
-            if(tst > mxv) {
-              mxv = tst;
-              mxa = f;
-              mxb = g;
-            }
-          }
-          g = b + 1;
-
-          for(f = a + 1; f < sf; f++) {
-            tst = score[f][g];
-            if(!((f == na) || (g == nb))) {
-              gap = (f - (a + 1));
-              if(gap)
-                tst += gap_penalty + ext_penalty * (gap - 1);
-            }
-            if(tst > mxv) {
-              mxv = tst;
-              mxa = f;
-              mxb = g;
-            }
-          }
-
-          if(max_skip) {
-            /* search for high scoring mismatched stretches */
-
-            sf = a + 1 + max_skip;
-            sg = b + 1 + max_skip;
-            if(sf > nf)
-              sf = nf;
-            if(sg > ng)
-              sg = ng;
-
-            for(f = a + 1; f < sf; f++) {
-              for(g = b + 1; g < sg; g++) {
-                tst = score[f][g];
-                /* only penalize if we are not at the end */
-                if(!((f == na) || (g == nb))) {
-                  gap = ((f - (a + 1)) + (g - (b + 1)));
-                  if(gap > 1)
-                    tst += 2 * gap_penalty + ext_penalty * (gap - 2);
-                }
-              }
-              if(tst > mxv) {
-                mxv = tst;
-                mxa = f;
-                mxb = g;
-              }
-            }
-          }
-
-          /* store what the best next step is */
-
-          point[a][b][0] = mxa;
-          point[a][b][1] = mxb;
-
-          /* and store the cumulative score for this cell */
-          score[a][b] = mxv + I->mat[a][b];
-
-          second_pass = true;
-        }
-      }
-    }
-
-    if(Feedback(G, FB_Match, FB_Debugging)) {
-      for(b = 0; b < nb; b++) {
-        for(a = 0; a < na; a++) {
-          printf("%4.1f(%2d,%2d)", score[a][b], point[a][b][0], point[a][b][1]);
-        }
-        printf("\n");
-      }
-    }
-
-    /* find the best entry point */
-
-    mxv = MIN_SCORE;
-    mxa = 0;
-    mxb = 0;
-    for(b = 0; b < nb; b++) {
-      for(a = 0; a < na; a++) {
-        tst = score[a][b];
-        if(tst > mxv) {
-          mxv = tst;
-          mxa = a;
-          mxb = b;
-        }
-      }
-    }
-    I->pair = VLAlloc(int, 2 * (na > nb ? na : nb));
-    p = I->pair;
-    a = mxa;
-    b = mxb;
-    cnt = 0;
-    while((a >= 0) && (b >= 0) && (a < na) && (b < nb)) {
-      *(p++) = a;
-      *(p++) = b;
-      f = point[a][b][0];
-      g = point[a][b][1];
-      a = f;
-      b = g;
-      cnt++;
-    }
-    PRINTFD(G, FB_Match)
-      " MatchAlign-DEBUG: best entry %8.3f %d %d %d\n", mxv, mxa, mxb, cnt ENDFD;
-    if(!quiet) {
-      PRINTFB(G, FB_Match, FB_Results)
-        " MatchAlign: score %1.3f\n", mxv ENDFD;
-    }
-    I->score = mxv;
-    I->n_pair = cnt;
-    VLASize(I->pair, int, (p - I->pair));
-    FreeP(score);
-    FreeP(point);
-  }
-  return (ok);
-}
-#endif
-
 void MatchFree(CMatch * I)
 {
   FreeP(I->da);
