@@ -283,14 +283,9 @@ static xx_word xx_matrix_decompose(xx_float64 * matrix, xx_word size,
       permute[j] = i_max;
       if(matrix[XX_MAT(size, j, j)] == 0.0) {
         /* here we have a choice: */
-#if 0
-        /* either (A): substitute a small value in place of machine-precision zero */
-        matrix[XX_MAT(size, j, j)] = 1e-20;
-#else
         /* or (B): fail outright */
         ok = false;
         break;
-#endif
       }
 
       if(j != (size - 1)) {
@@ -474,192 +469,6 @@ int MatrixTransformExtentsR44d3f(double *matrix,
   return 1;
 }
 
-#if 0
-
-
-/* currently unused */
-
-int MatrixInvertC44f(const float *m, float *out)
-{
-
-  /* This routine included in PyMOL under the terms of the 
-   * MIT consortium license for Brian Paul's Mesa, from which it was derived. */
-
-  /* MESA comments:
-   * Compute inverse of 4x4 transformation matrix.
-   * Code contributed by Jacques Leroy jle@star.be
-   * Return GL_TRUE for success, GL_FALSE for failure (singular matrix)
-   */
-
-  /* NB. OpenGL Matrices are COLUMN major. */
-#define SWAP_ROWS(a, b) { float *_tmp = a; (a)=(b); (b)=_tmp; }
-#define MAT(m,r,c) (m)[(c)*4+(r)]
-
-  float wtmp[4][8];
-  float m0, m1, m2, m3, s;
-  float *r0, *r1, *r2, *r3;
-
-  r0 = wtmp[0], r1 = wtmp[1], r2 = wtmp[2], r3 = wtmp[3];
-
-  r0[0] = MAT(m, 0, 0), r0[1] = MAT(m, 0, 1),
-    r0[2] = MAT(m, 0, 2), r0[3] = MAT(m, 0, 3),
-    r0[4] = 1.0, r0[5] = r0[6] = r0[7] = 0.0F,
-    r1[0] = MAT(m, 1, 0), r1[1] = MAT(m, 1, 1),
-    r1[2] = MAT(m, 1, 2), r1[3] = MAT(m, 1, 3),
-    r1[5] = 1.0, r1[4] = r1[6] = r1[7] = 0.0F,
-    r2[0] = MAT(m, 2, 0), r2[1] = MAT(m, 2, 1),
-    r2[2] = MAT(m, 2, 2), r2[3] = MAT(m, 2, 3),
-    r2[6] = 1.0, r2[4] = r2[5] = r2[7] = 0.0F,
-    r3[0] = MAT(m, 3, 0), r3[1] = MAT(m, 3, 1),
-    r3[2] = MAT(m, 3, 2), r3[3] = MAT(m, 3, 3), r3[7] = 1.0, r3[4] = r3[5] = r3[6] = 0.0F;
-
-  /* choose pivot - or die */
-  if(fabs(r3[0]) > fabs(r2[0]))
-    SWAP_ROWS(r3, r2);
-  if(fabs(r2[0]) > fabs(r1[0]))
-    SWAP_ROWS(r2, r1);
-  if(fabs(r1[0]) > fabs(r0[0]))
-    SWAP_ROWS(r1, r0);
-  if(0.0F == r0[0])
-    return 0;
-
-  /* eliminate first variable     */
-  m1 = r1[0] / r0[0];
-  m2 = r2[0] / r0[0];
-  m3 = r3[0] / r0[0];
-  s = r0[1];
-  r1[1] -= m1 * s;
-  r2[1] -= m2 * s;
-  r3[1] -= m3 * s;
-  s = r0[2];
-  r1[2] -= m1 * s;
-  r2[2] -= m2 * s;
-  r3[2] -= m3 * s;
-  s = r0[3];
-  r1[3] -= m1 * s;
-  r2[3] -= m2 * s;
-  r3[3] -= m3 * s;
-  s = r0[4];
-  if(s != 0.0F) {
-    r1[4] -= m1 * s;
-    r2[4] -= m2 * s;
-    r3[4] -= m3 * s;
-  }
-  s = r0[5];
-  if(s != 0.0F) {
-    r1[5] -= m1 * s;
-    r2[5] -= m2 * s;
-    r3[5] -= m3 * s;
-  }
-  s = r0[6];
-  if(s != 0.0F) {
-    r1[6] -= m1 * s;
-    r2[6] -= m2 * s;
-    r3[6] -= m3 * s;
-  }
-  s = r0[7];
-  if(s != 0.0F) {
-    r1[7] -= m1 * s;
-    r2[7] -= m2 * s;
-    r3[7] -= m3 * s;
-  }
-
-  /* choose pivot - or die */
-  if(fabs(r3[1]) > fabs(r2[1]))
-    SWAP_ROWS(r3, r2);
-  if(fabs(r2[1]) > fabs(r1[1]))
-    SWAP_ROWS(r2, r1);
-  if(0.0F == r1[1])
-    return 0;
-
-  /* eliminate second variable */
-  m2 = r2[1] / r1[1];
-  m3 = r3[1] / r1[1];
-  r2[2] -= m2 * r1[2];
-  r3[2] -= m3 * r1[2];
-  r2[3] -= m2 * r1[3];
-  r3[3] -= m3 * r1[3];
-  s = r1[4];
-  if(0.0F != s) {
-    r2[4] -= m2 * s;
-    r3[4] -= m3 * s;
-  }
-  s = r1[5];
-  if(0.0F != s) {
-    r2[5] -= m2 * s;
-    r3[5] -= m3 * s;
-  }
-  s = r1[6];
-  if(0.0F != s) {
-    r2[6] -= m2 * s;
-    r3[6] -= m3 * s;
-  }
-  s = r1[7];
-  if(0.0F != s) {
-    r2[7] -= m2 * s;
-    r3[7] -= m3 * s;
-  }
-
-  /* choose pivot - or die */
-  if(fabs(r3[2]) > fabs(r2[2]))
-    SWAP_ROWS(r3, r2);
-  if(0.0F == r2[2])
-    return 0;
-
-  /* eliminate third variable */
-  m3 = r3[2] / r2[2];
-  r3[3] -= m3 * r2[3], r3[4] -= m3 * r2[4],
-    r3[5] -= m3 * r2[5], r3[6] -= m3 * r2[6], r3[7] -= m3 * r2[7];
-
-  /* last check */
-  if(0.0F == r3[3])
-    return 0;
-
-  s = 1.0F / r3[3];             /* now back substitute row 3 */
-  r3[4] *= s;
-  r3[5] *= s;
-  r3[6] *= s;
-  r3[7] *= s;
-
-  m2 = r2[3];                   /* now back substitute row 2 */
-  s = 1.0F / r2[2];
-  r2[4] = s * (r2[4] - r3[4] * m2), r2[5] = s * (r2[5] - r3[5] * m2),
-    r2[6] = s * (r2[6] - r3[6] * m2), r2[7] = s * (r2[7] - r3[7] * m2);
-  m1 = r1[3];
-  r1[4] -= r3[4] * m1, r1[5] -= r3[5] * m1, r1[6] -= r3[6] * m1, r1[7] -= r3[7] * m1;
-  m0 = r0[3];
-  r0[4] -= r3[4] * m0, r0[5] -= r3[5] * m0, r0[6] -= r3[6] * m0, r0[7] -= r3[7] * m0;
-
-  m1 = r1[2];                   /* now back substitute row 1 */
-  s = 1.0F / r1[1];
-  r1[4] = s * (r1[4] - r2[4] * m1), r1[5] = s * (r1[5] - r2[5] * m1),
-    r1[6] = s * (r1[6] - r2[6] * m1), r1[7] = s * (r1[7] - r2[7] * m1);
-  m0 = r0[2];
-  r0[4] -= r2[4] * m0, r0[5] -= r2[5] * m0, r0[6] -= r2[6] * m0, r0[7] -= r2[7] * m0;
-
-  m0 = r0[1];                   /* now back substitute row 0 */
-  s = 1.0F / r0[0];
-  r0[4] = s * (r0[4] - r1[4] * m0), r0[5] = s * (r0[5] - r1[5] * m0),
-    r0[6] = s * (r0[6] - r1[6] * m0), r0[7] = s * (r0[7] - r1[7] * m0);
-
-  MAT(out, 0, 0) = r0[4];
-  MAT(out, 0, 1) = r0[5], MAT(out, 0, 2) = r0[6];
-  MAT(out, 0, 3) = r0[7], MAT(out, 1, 0) = r1[4];
-  MAT(out, 1, 1) = r1[5], MAT(out, 1, 2) = r1[6];
-  MAT(out, 1, 3) = r1[7], MAT(out, 2, 0) = r2[4];
-  MAT(out, 2, 1) = r2[5], MAT(out, 2, 2) = r2[6];
-  MAT(out, 2, 3) = r2[7], MAT(out, 3, 0) = r3[4];
-  MAT(out, 3, 1) = r3[5], MAT(out, 3, 2) = r3[6];
-  MAT(out, 3, 3) = r3[7];
-
-  return 1;
-
-#undef MAT
-#undef SWAP_ROWS
-}
-#endif
-
-
 /*========================================================================*/
 int *MatrixFilter(float cutoff, int window, int n_pass, int nv, float *v1, float *v2)
 {
@@ -687,8 +496,6 @@ int *MatrixFilter(float cutoff, int window, int n_pass, int nv, float *v1, float
     copy3f(v2, center2);
     for(a = 1; a < nv; a++) {
       if(flag[a]) {
-        vv1 = v1 + 3 * a;
-        vv2 = v2 + 3 * a;
         add3f(v1, center1, center1);
         add3f(v2, center2, center2);
         cc++;
@@ -1060,6 +867,20 @@ float MatrixFitRMSTTTf(PyMOLGlobals * G, int n, float *v1, float *v2, float *wt,
   int a, b, c, maxiter;
   double t1[3], t2[3];
 
+  /* special case: just one atom pair */
+
+  if(n == 1) {
+    if(ttt) {
+      for(a = 1; a < 11; a++)
+        ttt[a] = 0.0F;
+      for(a = 0; a < 12; a += 5)
+        ttt[a] = 1.0F;
+      for(a = 0; a < 3; a++)
+        ttt[a + 12] = v2[a] - v1[a];
+    }
+    return 0.0F;
+  }
+
   /* Initialize arrays. */
 
   for(a = 0; a < 3; a++) {
@@ -1071,8 +892,8 @@ float MatrixFitRMSTTTf(PyMOLGlobals * G, int n, float *v1, float *v2, float *wt,
   }
 
   sumwt = 0.0F;
-  tol = SettingGet(G, cSetting_fit_tolerance);
-  maxiter = (int) SettingGet(G, cSetting_fit_iterations);
+  tol = SettingGetGlobal_f(G, cSetting_fit_tolerance);
+  maxiter = SettingGetGlobal_i(G, cSetting_fit_iterations);
 
   /* Calculate center-of-mass vectors */
 
@@ -1693,11 +1514,8 @@ double d_sign(doublereal * a, doublereal * b)
 
 static doublereal c_b126 = 0.;
 
-static int balanc_(nm, n, a, low, igh, scale)
-     integer *nm, *n;
-     doublereal *a;
-     integer *low, *igh;
-     doublereal *scale;
+static int balanc_(integer * nm, integer * n, doublereal * a, integer * low,
+                       integer * igh, doublereal * scale)
 {
 
 
@@ -2046,11 +1864,8 @@ L280:
   return 0;
 }                               /* balanc_ */
 
-static int balbak_(nm, n, low, igh, scale, m, z__)
-     integer *nm, *n, *low, *igh;
-     doublereal *scale;
-     integer *m;
-     doublereal *z__;
+static int balbak_(integer * nm, integer * n, integer * low, integer * igh,
+                   doublereal * scale, integer * m, doublereal * z__)
 {
 
 
@@ -2215,8 +2030,8 @@ L200:
   return 0;
 }                               /* balbak_ */
 
-static int cdiv_(ar, ai, br, bi, cr, ci)
-     doublereal *ar, *ai, *br, *bi, *cr, *ci;
+static int cdiv_(doublereal * ar, doublereal * ai, doublereal * br, doublereal * bi,
+                 doublereal * cr, doublereal * ci)
 {
 
 
@@ -2249,10 +2064,8 @@ static int cdiv_(ar, ai, br, bi, cr, ci)
   return 0;
 }                               /* cdiv_ */
 
-static int elmhes_(nm, n, low, igh, a, int__)
-     integer *nm, *n, *low, *igh;
-     doublereal *a;
-     integer *int__;
+static int elmhes_(integer * nm, integer * n, integer * low, integer * igh,
+                   doublereal * a, integer * int__)
 {
 
 
@@ -2453,11 +2266,8 @@ L200:
   return 0;
 }                               /* elmhes_ */
 
-static int eltran_(nm, n, low, igh, a, int__, z__)
-     integer *nm, *n, *low, *igh;
-     doublereal *a;
-     integer *int__;
-     doublereal *z__;
+static int eltran_(integer * nm, integer * n, integer * low, integer * igh,
+                   doublereal * a, integer * int__, doublereal * z__)
 {
   /* System generated locals */
   integer a_dim1, a_offset, z_dim1, z_offset, i__1, i__2;
@@ -2621,10 +2431,8 @@ L200:
   return 0;
 }                               /* eltran_ */
 
-static int hqr_(nm, n, low, igh, h__, wr, wi, ierr)
-     integer *nm, *n, *low, *igh;
-     doublereal *h__, *wr, *wi;
-     integer *ierr;
+static int hqr_(integer * nm, integer * n, integer * low, integer * igh,
+                doublereal * h__, doublereal * wr, doublereal * wi, integer * ierr)
 {
   /* System generated locals */
   integer h_dim1, h_offset, i__1, i__2, i__3;
@@ -3094,10 +2902,9 @@ L1001:
   return 0;
 }                               /* hqr_ */
 
-static int hqr2_(nm, n, low, igh, h__, wr, wi, z__, ierr)
-     integer *nm, *n, *low, *igh;
-     doublereal *h__, *wr, *wi, *z__;
-     integer *ierr;
+static int hqr2_(integer * nm, integer * n, integer * low, integer * igh,
+                 doublereal * h__, doublereal * wr, doublereal * wi, doublereal * z__,
+                 integer * ierr)
 {
   /* System generated locals */
   integer h_dim1, h_offset, z_dim1, z_offset, i__1, i__2, i__3;
@@ -3999,16 +3806,6 @@ L1001:
 int pymol_rg_(integer * nm, integer * n, doublereal * a, doublereal * wr,
               doublereal * wi, integer * matz, doublereal * z__, integer * iv1,
               doublereal * fv1, integer * ierr)
-#if 0
-     integer *nm, *n;
-     doublereal *a, *wr, *wi;
-     integer *matz;
-     doublereal *z__;
-     integer *iv1;
-     doublereal *fv1;
-     integer *ierr;
-#endif
-
 {
   /* System generated locals */
   integer a_dim1, a_offset, z_dim1, z_offset;
