@@ -1,9 +1,5 @@
 #ifndef _PyMOL_VERSION
-#ifndef PYMOL_EVAL
-#define _PyMOL_VERSION "1.5.0.1"
-#else
-#define _PyMOL_VERSION "1.5.0.1 eval"
-#endif
+#define _PyMOL_VERSION "1.7.0.0"
 #endif
 
 #ifndef _PYMOL_1_x
@@ -11,12 +7,12 @@
 #endif
 
 #ifndef _PyMOL_VERSION_double
-#define _PyMOL_VERSION_double 1.501
+#define _PyMOL_VERSION_double 1.700
 #endif
 
 
 /* for session file compatibility */
 
 #ifndef _PyMOL_VERSION_int
-#define _PyMOL_VERSION_int 1501
+#define _PyMOL_VERSION_int 1700
 #endif
