@@ -83,7 +83,7 @@ static void lookup_color(CColor * I, float *in, float *out, int big_endian);
 
 void ColorGetBkrdContColor(PyMOLGlobals * G, float *rgb, int invert_flag)
 {
-  float *bkrd = SettingGetfv(G, cSetting_bg_rgb);
+  float *bkrd = ColorGet(G, SettingGet_color(G, NULL, NULL, cSetting_bg_rgb));
 
   if(!invert_flag) {
     if((bkrd[0] + bkrd[1] + bkrd[2]) > 0.5F) {
@@ -144,7 +144,7 @@ int ColorGetNext(PyMOLGlobals * G)
 {
   int result;
   int next;
-  next = (int) SettingGet(G, cSetting_auto_color_next);
+  next = SettingGetGlobal_i(G, cSetting_auto_color_next);
 
   if(next >= nAutoColor)
     next = 0;
@@ -160,7 +160,7 @@ int ColorGetCurrent(PyMOLGlobals * G)
 {
   int result;
   int next;
-  next = (int) SettingGet(G, cSetting_auto_color_next);
+  next = SettingGetGlobal_i(G, cSetting_auto_color_next);
   next--;
   if(next < 0)
     next = (nAutoColor - 1);
@@ -267,6 +267,7 @@ void ColorRegisterExt(PyMOLGlobals * G, char *name, void *ptr, int type)
     {
       OVreturn_word result = OVLexicon_GetFromCString(I->Lex, name);
       if(OVreturn_IS_OK(result)) {
+        OVOneToOne_Set(I->Idx, result.word, cColorExtCutoff - a);
         I->Ext[a].Name = result.word;
       } else {
         I->Ext[a].Name = 0;
@@ -290,6 +291,7 @@ void ColorForgetExt(PyMOLGlobals * G, char *name)
       OVLexicon_DecRef(I->Lex, I->Ext[a].Name);
       OVOneToOne_DelForward(I->Idx, I->Ext[a].Name);
     }
+    I->Ext[a].Name = 0;
     I->Ext[a].Ptr = NULL;
   }
 }
@@ -363,30 +365,6 @@ PyObject *ColorAsPyList(PyMOLGlobals * G)
 #endif
 }
 
-#if 0
-
-/*========================================================================*/
-PyObject *ColorTableAsPyList(PyMOLGlobals * G)
-{
-#ifdef _PYMOL_NOPY
-  return NULL;
-#else
-  register CColor *I = G->Color;
-  PyObject *result;
-
-  result = PyList_New(2);
-  PyList_SetItem(result, 0, PyFloat_FromDouble(I->Gamma));
-  if(0 && I->ColorTable) {      /* for now, we don't embed the color table due to size... */
-    PyList_SetItem(result, 1, PConvIntArrayToPyList((int *) (I->ColorTable), 512 * 512));
-  } else {
-    PyList_SetItem(result, 1, PConvAutoNone(Py_None));
-  }
-  return result;
-#endif
-}
-#endif
-
-
 /*========================================================================*/
 int ColorConvertOldSessionIndex(PyMOLGlobals * G, int index)
 {
@@ -471,7 +449,7 @@ int ColorExtFromPyList(PyMOLGlobals * G, PyObject * list, int partial_restore)
         OVreturn_word result;
         ok = PConvPyStrToStr(PyList_GetItem(rec, 0), name, sizeof(WordType));
         if(OVreturn_IS_OK(result = OVLexicon_GetFromCString(I->Lex, name))) {
-          OVOneToOne_Set(I->Idx, result.word, a);
+          OVOneToOne_Set(I->Idx, result.word, cColorExtCutoff - a);
           ext->Name = result.word;
         } else {
           ext->Name = 0;
@@ -589,53 +567,6 @@ int ColorFromPyList(PyMOLGlobals * G, PyObject * list, int partial_restore)
 #endif
 }
 
-
-/*========================================================================*/
-#if 0
-int ColorTableFromPyList(PyMOLGlobals * G, PyObject * list, int partial_restore)
-{
-#ifdef _PYMOL_NOPY
-  return 0;
-#else
-  register CColor *I = G->Color;
-  int ok = true;
-  int ll = 0;
-  if(!partial_restore) {
-    if(ok)
-      ok = (list != NULL);
-    if(ok)
-      ok = PyList_Check(list);
-    if(ok) {
-      ll = PyList_Size(list);
-      if(ll > 1) {
-        float tmp_float;
-        if(ok)
-          ok = PConvPyFloatToFloat(PyList_GetItem(list, 0), &tmp_float);
-        if(ok)
-          I->Gamma = tmp_float;
-        {
-          PyObject *tmp = PyList_GetItem(list, 1);
-          if(tmp && (tmp != Py_None)) {
-#if 0
-            int PConvPyListToIntArrayInPlace(PyObject * obj, int *ff, ov_size ll);
-
-            int PConvPyListToIntArrayInPlace(tmp, int *ff, ov_size ll);
-            PyList_SetItem(list, 2, PConvIntArrayToPyList(I->ColorTable, 512 * 512));
-
-#endif
-
-            ColorUpdateFromLut(G, -1);
-          }
-        }
-      }
-    }
-  }
-  return ok;
-#endif
-}
-#endif
-
-
 /*========================================================================*/
 void ColorDef(PyMOLGlobals * G, char *name, float *v, int mode, int quiet)
 {
@@ -753,6 +684,8 @@ int ColorGetIndex(PyMOLGlobals * G, char *name)
         return cColorBack;
       else if(i == -1)
         return -1;
+      if (i & cColor_TRGB_Bits)
+        return i;
     }
   }
   if((name[0] == '0') && (name[1] == 'x')) {    /* explicit hex RGB 0x000000 */
@@ -803,9 +736,9 @@ int ColorGetIndex(PyMOLGlobals * G, char *name)
       }
     }
     if(best || (color < 0)) {
-      ext_color = ColorFindExtByName(G, name, false, &ext_best);
+      ext_color = ColorFindExtByName(G, name, true, &ext_best);
       if(ext_color >= 0) {
-        ext_color = -10 - ext_color;    /* indicates external */
+        ext_color = cColorExtCutoff - ext_color;    /* indicates external */
         if((!ext_best) || (ext_best > best))    /* perfect or better match? */
           color = ext_color;
       }
@@ -832,9 +765,9 @@ char *ColorGetName(PyMOLGlobals * G, int index)
     index = (((index & 0xFFFFFF) | ((index << 2) & 0xFC000000) |        /* convert 6 bits of trans into 8 */
               ((index >> 4) & 0x03000000)));
     if(index & 0xFF000000)      /* if transparent */
-      sprintf(I->RGBName, "%08x", index);
+      sprintf(I->RGBName, "0x%08x", index);
     else                        /* else */
-      sprintf(I->RGBName, "%06x", index);
+      sprintf(I->RGBName, "0x%06x", index);
     return I->RGBName;
   } else if(index <= cColorExtCutoff) {
     int a = cColorExtCutoff - index;
@@ -2642,10 +2575,10 @@ int ColorTableLoad(PyMOLGlobals * G, char *fname, float gamma, int quiet)
 
       float min_factor = 0.15F;
 
-      red_max = SettingGet(G, cSetting_pymol_space_max_red);
-      green_max = SettingGet(G, cSetting_pymol_space_max_green);
-      blue_max = SettingGet(G, cSetting_pymol_space_max_blue);
-      min_factor = SettingGet(G, cSetting_pymol_space_min_factor);
+      red_max = SettingGetGlobal_f(G, cSetting_pymol_space_max_red);
+      green_max = SettingGetGlobal_f(G, cSetting_pymol_space_max_green);
+      blue_max = SettingGetGlobal_f(G, cSetting_pymol_space_max_blue);
+      min_factor = SettingGetGlobal_f(G, cSetting_pymol_space_min_factor);
 
       FreeP(I->ColorTable);
       if(I->BigEndian)
@@ -2787,24 +2720,6 @@ int ColorTableLoad(PyMOLGlobals * G, char *fname, float gamma, int quiet)
   return (ok);
 }
 
-#if 0
-static void unlookup_color(unsigned int *table, float *in, float *out, int big_endian)
-{
-  /* simple iterative approach to finding closest input for a desired
-     output given the current color mapping table */
-
-  float cur_in[3], cur_out[3], diff[3];
-  copy3f(in, cur_in);
-  for(a = 0; a < 10; a++) {
-    lookup_color(table, cur_in, cur_out, big_endian);
-    diff3f(cur_out, in, diff);
-    scale3f(0.5, diff, diff);
-    subtract3f(cur_in, diff, cur_in);
-  }
-  copy3f(cur_in, out);
-}
-#endif
-
 static void lookup_color(CColor * I, float *in, float *out, int big_endian)
 {
   const float _1 = 1.0F;
@@ -3007,8 +2922,8 @@ int ColorInit(PyMOLGlobals * G)
     testPtr = (unsigned char *) &test;
     I->BigEndian = (*testPtr) && 1;
 
-    I->Color = VLAMalloc(5500, sizeof(ColorRec), 5, true);
-    I->Ext = VLAMalloc(2, sizeof(ExtRec), 5, true);
+    I->Color = VLACalloc(ColorRec, 5500);
+    I->Ext = VLACalloc(ExtRec, 2);
     I->Gamma = 1.0F;
 
     ColorReset(G);              /* will alloc I->Idx and I->Lex */
@@ -3032,6 +2947,27 @@ void ColorUpdateFront(PyMOLGlobals * G, float *back)
     zero3f(I->Front);
 }
 
+void ColorUpdateFrontFromSettings(PyMOLGlobals * G){
+  int bg_gradient = SettingGet_b(G, NULL, NULL, cSetting_bg_gradient);
+  OrthoLineType bg_image_filename;
+  strcpy (bg_image_filename, SettingGet_s(G, NULL, NULL, cSetting_bg_image_filename));
+  
+  if (!bg_gradient){
+    if (!bg_image_filename[0] && !OrthoBackgroundDataIsSet(G)){
+      float *v = ColorGet(G, SettingGet_color(G, NULL, NULL, cSetting_bg_rgb));
+      ColorUpdateFront(G, v);
+    } else {
+      float v[] = { 0.f, 0.f, 0.f };
+      ColorUpdateFront(G, v);
+    }
+  } else {
+    float vv[3], *v = ColorGet(G, SettingGet_color(G, NULL, NULL, cSetting_bg_rgb_bottom));
+    float *vb = ColorGet(G, SettingGet_color(G, NULL, NULL, cSetting_bg_rgb_top));
+    average3f(v, vb, vv);
+    ColorUpdateFront(G, vv);    
+  }
+}
+
 
 /*========================================================================*/
 float *ColorGetSpecial(PyMOLGlobals * G, int index)
@@ -3127,3 +3063,15 @@ int ColorGetEncoded(PyMOLGlobals * G, int index, float *color)
   }
   return 1;
 }
+
+int Color3fToInt(PyMOLGlobals * G, float *rgb){
+  unsigned int rc, gc, bc;
+  unsigned int result;
+  rc = pymol_roundf(rgb[0] * 255.);
+  gc = pymol_roundf(rgb[1] * 255.);
+  bc = pymol_roundf(rgb[2] * 255.);
+  return ( ( cColor_TRGB_Bits & 0xFF000000) | 
+	   ( ( rc << 16 ) & 0x00FF0000) |
+	   ( ( gc << 8 ) & 0x0000FF00) |
+	   ( ( bc & 0x000000FF ) ) );
+}
