@@ -37,6 +37,8 @@ Z* -------------------------------------------------------------------
 #include"PyMOL.h"
 #include"ScrollBar.h"
 #include"Menu.h"
+#include"View.h"
+#include"Seq.h"
 
 #define cMovieDragModeMoveKey   1
 #define cMovieDragModeInsDel    2
@@ -251,8 +253,8 @@ void MovieCopyPrepare(PyMOLGlobals * G, int *width, int *height, int *length)
   register CMovie *I = G->Movie;
   int nFrame;
 
-  I->CacheSave = (int) SettingGet(G, cSetting_cache_frames);
-  I->OverlaySave = (int) SettingGet(G, cSetting_overlay);
+  I->CacheSave = SettingGetGlobal_b(G, cSetting_cache_frames);
+  I->OverlaySave = SettingGetGlobal_i(G, cSetting_overlay);
   if(!I->CacheSave)
     MovieClearImages(G);
   SettingSet(G, cSetting_cache_frames, 1.0);
@@ -624,7 +626,7 @@ void MoviePlay(PyMOLGlobals * G, int cmd)
   switch (cmd) {
   case cMovieToggle:
     I->Playing = !I->Playing;
-    if(I->Playing && !(int) SettingGet(G, cSetting_movie_loop)) {
+    if(I->Playing && !SettingGetGlobal_b(G, cSetting_movie_loop)) {
       /* if not looping, and at end of movie, then automatically rewind
          and force execution of the first movie command */
       if((SettingGetGlobal_i(G, cSetting_frame)) == (SceneGetNFrame(G, NULL))) {
@@ -636,7 +638,7 @@ void MoviePlay(PyMOLGlobals * G, int cmd)
     I->Playing = false;
     break;
   case cMoviePlay:
-    if(!(int) SettingGet(G, cSetting_movie_loop)) {
+    if(!SettingGetGlobal_b(G, cSetting_movie_loop)) {
       /* if not looping, and at end of movie, then automatically rewind
          and force execution of the first movie command */
       if((SettingGetGlobal_i(G, cSetting_frame)) == (SceneGetNFrame(G, NULL))) {
@@ -694,7 +696,7 @@ static void MovieModalPNG(PyMOLGlobals * G, CMovie * I, CMovieModal * M)
   switch (M->stage) {
   case 0:                      /* setup */
     MovieSetRealtime(G, false);
-    M->save = (int) SettingGet(G, cSetting_cache_frames);
+    M->save = SettingGetGlobal_b(G, cSetting_cache_frames);
     if(!M->save)
       MovieClearImages(G);
     SettingSet(G, cSetting_cache_frames, 1.0);
@@ -907,6 +909,15 @@ int MoviePNG(PyMOLGlobals * G, char *prefix, int save, int start,
   M->mode = mode;
   M->quiet = quiet;
 
+  if(SettingGetGlobal_b(G, cSetting_seq_view)) {
+    PRINTFB(G, FB_Movie, FB_Warnings)
+      " MoviePNG-Warning: disabling seq_view, may conflict with movie export\n" ENDFB(G);
+    SettingSetGlobal_b(G, cSetting_seq_view, 0);
+    // force viewport update
+    SeqChanged(G);
+    OrthoDoDraw(G, 0);
+  }
+
   if(modal < 0) {
     /* default behavior is to go modal unless we're ray tracing */
     if((mode < cSceneImage_Ray) || (!SettingGetGlobal_b(G, cSetting_ray_trace_frames))) {
@@ -1009,7 +1020,7 @@ void MovieAppendSequence(PyMOLGlobals * G, char *str, int start_from,int freeze)
 int MovieFrameToImage(PyMOLGlobals * G, int frame)
 {
   int result = 0;
-  int single_image = (int) SettingGet(G, cSetting_single_image);
+  int single_image = SettingGetGlobal_b(G, cSetting_single_image);
   if(single_image)
     result = MovieFrameToIndex(G, frame);
   else
@@ -1616,6 +1627,10 @@ static int MovieClick(Block * block, int button, int x, int y, int mod)
         break;
       default:
         ScrollBarDoClick(I->ScrollBar, button, x, y, mod);
+        {
+	  float scroll_value = ScrollBarGetValue(I->ScrollBar);
+	  SceneSetFrame(G, 7, scroll_value);
+	}
         break;
       }
     }
@@ -1794,15 +1809,25 @@ int MovieGetPanelHeight(PyMOLGlobals * G)
   }
 }
 
-void MovieDrawViewElem(PyMOLGlobals *G, BlockRect *rect,int frames)
+void MovieDrawViewElem(PyMOLGlobals *G, BlockRect *rect,int frames ORTHOCGOARG)
 {
   CMovie *I = G->Movie;
   if(I->ViewElem) {
-    ViewElemDraw(G,I->ViewElem,rect,frames,"camera");
+    ViewElemDraw(G,I->ViewElem,rect,frames,"camera" ORTHOCGOARGVAR);
   }
 }
 
-static void MovieDraw(Block * block)
+short MovieFastDraw(Block * block ORTHOCGOARG)
+{
+  PyMOLGlobals *G = block->G;
+  CMovie *I = G->Movie;
+  //  ScrollBarDrawHandle(I->ScrollBar, 0.35F ORTHOCGOARGVAR);
+  ScrollBarDoDrawNoFill(I->ScrollBar ORTHOCGOARGVAR);
+  ScrollBarDrawHandle(I->ScrollBar, 0.35F ORTHOCGOARGVAR);
+  return true;
+}
+
+static void MovieDraw(Block * block ORTHOCGOARG)
 {
   PyMOLGlobals *G = block->G;
   CMovie *I = G->Movie;
@@ -1816,33 +1841,23 @@ static void MovieDraw(Block * block)
 
       if(G->HaveGUI && G->ValidContext) {
         float black[3] = {0.0F,0.0F,0.0F};
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-#ifdef _PYMOL_GL_DRAWARRAYS
-	  {
-	    const GLfloat polyVerts[] = {
-	      rect.right, rect.bottom,
-	      rect.right, rect.top,
-	      block->rect.right, rect.bottom,
-	      block->rect.right, rect.top
-	    };
-	    glColor4f(black[0], black[1], black[2], 1.F);	
-	    glEnableClientState(GL_VERTEX_ARRAY);
-	    glVertexPointer(2, GL_FLOAT, 0, polyVerts);
-	    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-	    glDisableClientState(GL_VERTEX_ARRAY);
-	  }
-#else
-        glColor3fv(black);
-        glBegin(GL_POLYGON);
-        glVertex2f(rect.right, rect.bottom);
-        glVertex2f(rect.right, rect.top);
-        glVertex2f(block->rect.right, rect.top);
-        glVertex2f(block->rect.right, rect.bottom);
-        glEnd();
-#endif
-#endif
+	if (orthoCGO){
+	  CGOColorv(orthoCGO, black);
+	  CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+	  CGOVertex(orthoCGO, rect.right, rect.bottom, 0.f);
+	  CGOVertex(orthoCGO, rect.right, rect.top, 0.f);
+	  CGOVertex(orthoCGO, block->rect.right, rect.bottom, 0.f);
+	  CGOVertex(orthoCGO, block->rect.right, rect.top, 0.f);
+	  CGOEnd(orthoCGO);
+	} else {
+	  glColor3fv(black);
+	  glBegin(GL_POLYGON);
+	  glVertex2f(rect.right, rect.bottom);
+	  glVertex2f(rect.right, rect.top);
+	  glVertex2f(block->rect.right, rect.top);
+	  glVertex2f(block->rect.right, rect.bottom);
+	  glEnd();
+	}
       }
 
       if(!n_frame) {
@@ -1863,9 +1878,14 @@ static void MovieDraw(Block * block)
       }
       ScrollBarSetBox(I->ScrollBar, rect.top,
                       rect.left, rect.bottom, rect.right);
-      ScrollBarDoDraw(I->ScrollBar);
-      ExecutiveMotionDraw(G,&rect,count);
-      ScrollBarDrawHandle(I->ScrollBar, 0.35F);
+      if (orthoCGO){
+	ScrollBarFill(I->ScrollBar ORTHOCGOARGVAR);
+	ExecutiveMotionDraw(G,&rect,count ORTHOCGOARGVAR);
+      } else {
+	ScrollBarDoDraw(I->ScrollBar ORTHOCGOARGVAR);
+	ExecutiveMotionDraw(G,&rect,count ORTHOCGOARGVAR);
+	ScrollBarDrawHandle(I->ScrollBar, 0.35F ORTHOCGOARGVAR);
+      }
 
       /* drag selection box */
       if(I->DragDraw) {
@@ -1878,9 +1898,9 @@ static void MovieDraw(Block * block)
           {
             float grey[4] = {0.75F,0.75F,0.75f,0.5};
             if(I->DragStartFrame<n_frame) 
-              ViewElemDrawBox(G,&I->DragRect, I->DragStartFrame, I->DragStartFrame+1, n_frame, white, false);        
+              ViewElemDrawBox(G,&I->DragRect, I->DragStartFrame, I->DragStartFrame+1, n_frame, white, false ORTHOCGOARGVAR);        
             if((I->DragCurFrame>=0) && (I->DragCurFrame<n_frame)) {
-              ViewElemDrawBox(G,&I->DragRect, I->DragCurFrame, I->DragCurFrame+1, n_frame, grey, true);
+              ViewElemDrawBox(G,&I->DragRect, I->DragCurFrame, I->DragCurFrame+1, n_frame, grey, true ORTHOCGOARGVAR);
             }
           }
           break;
@@ -1894,19 +1914,19 @@ static void MovieDraw(Block * block)
             if(max_frame<0) max_frame = 0;
             if(min_frame>=n_frame) min_frame = n_frame - 1;
             if(max_frame>=n_frame) max_frame = n_frame - 1;
-            ViewElemDrawBox(G,&I->DragRect, min_frame, max_frame+1, n_frame, white, false);        
-            ViewElemDrawBox(G,&I->DragRect, min_frame, max_frame+1, n_frame, grey, true);
+            ViewElemDrawBox(G,&I->DragRect, min_frame, max_frame+1, n_frame, white, false ORTHOCGOARGVAR);        
+            ViewElemDrawBox(G,&I->DragRect, min_frame, max_frame+1, n_frame, grey, true ORTHOCGOARGVAR);
           }
           break;
         case cMovieDragModeInsDel:
           if(I->DragCurFrame==I->DragStartFrame) {
-            ViewElemDrawBox(G,&I->DragRect, I->DragStartFrame, I->DragStartFrame, n_frame, white, true);        
+            ViewElemDrawBox(G,&I->DragRect, I->DragStartFrame, I->DragStartFrame, n_frame, white, true ORTHOCGOARGVAR);        
           } else if(I->DragCurFrame>=I->DragStartFrame) {
             float green[4] = {0.5F, 1.0F, 0.5F,0.5F};
-            ViewElemDrawBox(G,&I->DragRect, I->DragStartFrame, I->DragCurFrame, n_frame, green, true);        
+            ViewElemDrawBox(G,&I->DragRect, I->DragStartFrame, I->DragCurFrame, n_frame, green, true ORTHOCGOARGVAR);        
           } else {
             float red[4] = {1.0F, 0.5F, 0.5F,0.5F};          
-            ViewElemDrawBox(G,&I->DragRect, I->DragCurFrame, I->DragStartFrame, n_frame, red, true);        
+            ViewElemDrawBox(G,&I->DragRect, I->DragCurFrame, I->DragStartFrame, n_frame, red, true ORTHOCGOARGVAR);        
           }
           break;
         }
@@ -1951,13 +1971,14 @@ int MovieInit(PyMOLGlobals * G)
     I->Block->fClick = MovieClick;
     I->Block->fDrag = MovieDrag;
     I->Block->fDraw = MovieDraw;
+    I->Block->fFastDraw = MovieFastDraw;
     I->Block->fReshape = MovieReshape;
     I->Block->active = true;
     I->ScrollBar = ScrollBarNew(G, true);
     OrthoAttach(G, I->Block, cOrthoTool);
 
     I->Playing = false;
-    I->Image = VLAMalloc(10, sizeof(ImageType), 5, true);       /* auto-zero */
+    I->Image = VLACalloc(ImageType *, 10);       /* auto-zero */
     I->Sequence = NULL;
     I->Cmd = NULL;
     I->ViewElem = NULL;
