@@ -34,11 +34,6 @@ the initialization functions for these libraries on startup.
 #ifdef WIN32
 #include<windows.h>
 #include<process.h>
-#ifndef WIN64
-#if 0
-#include<winappc.h>
-#endif
-#endif
 #endif
 
 /* END PROPRIETARY CODE SEGMENT */
@@ -438,8 +433,6 @@ static void PLockAPIWhileBlocked(PyMOLGlobals * G);
 
 #define xxxPYMOL_NEW_THREADS 1
 
-unsigned int PyThread_get_thread_ident(void);   /* critical functionality */
-
 void PRunStringModule(PyMOLGlobals * G, char *str);
 
 void PLockStatus(PyMOLGlobals * G)
@@ -800,7 +793,7 @@ void PDumpException()
   PyObject_CallMethod(P_traceback, "print_exc", "");
 }
 
-int PAlterAtomState(PyMOLGlobals * G, float *v, char *expr, int read_only,
+int PAlterAtomState(PyMOLGlobals * G, float *v, PyCodeObject *expr_co, int read_only,
                     AtomInfoType * at, char *model, int index, PyObject * space)
 
 /* assumes Blocked python interpreter */
@@ -873,7 +866,7 @@ int PAlterAtomState(PyMOLGlobals * G, float *v, char *expr, int read_only,
   x_id1 = PConvFloatToPyDictItem(dict, "x", v[0]);
   y_id1 = PConvFloatToPyDictItem(dict, "y", v[1]);
   z_id1 = PConvFloatToPyDictItem(dict, "z", v[2]);
-  PXDecRef(PyRun_String(expr, Py_single_input, space, dict));
+  PXDecRef(PyEval_EvalCode(expr_co, space, dict));
   if(PyErr_Occurred()) {
     PyErr_Print();
     result = false;
@@ -923,7 +916,7 @@ int PAlterAtomState(PyMOLGlobals * G, float *v, char *expr, int read_only,
 }
 
 int PAlterAtom(PyMOLGlobals * G,
-               AtomInfoType * at, char *expr, int read_only,
+               AtomInfoType * at, PyCodeObject *expr_co, int read_only,
                char *model, int index, PyObject * space)
 {
   /* assumes Blocked python interpreter */
@@ -1041,7 +1034,7 @@ int PAlterAtom(PyMOLGlobals * G,
   state_id1 = PConvIntToPyDictItem(dict, "state", at->discrete_state);
   rank_id1 = PConvIntToPyDictItem(dict, "rank", at->rank);
 
-  PXDecRef(PyRun_String(expr, Py_single_input, space, dict));
+  PXDecRef(PyEval_EvalCode(expr_co, space, dict));
   if(PyErr_Occurred()) {
     ErrMessage(G, "Alter", "Aborting on error. Assignment may be incomplete.");
     PyErr_Print();
@@ -1588,18 +1581,19 @@ int PLockAPIAsGlut(PyMOLGlobals * G, int block_if_busy)
 /* THESE CALLS ARE REQUIRED FOR MONOLITHIC COMPILATION TO SUCCEED UNDER WINDOWS. */
 #ifndef _PYMOL_EMBEDDED
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* 
  *  void        initExtensionClass(void);
  *   void        initsglite(void);
  */
 void init_champ(void);
-void init_opengl(void);
-void init_opengl_num(void);
-void init_glu(void);
-void init_glu_num(void);
-void init_glut(void);
-void initopenglutil(void);
-void initopenglutil_num(void);
+
+#ifdef __cplusplus
+}
+#endif
 #endif
 
 #ifdef _PYMOL_MONOLITHIC
@@ -1623,6 +1617,9 @@ void initranlib();
 
 #ifdef _PYMOL_MONOLITHIC
 #ifndef _PYMOL_EMBEDDED
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /*
  * void        initExtensionClass(void);
@@ -1632,13 +1629,10 @@ void init_champ(void);
 #ifdef _PYMOL_PYOMM
 void init_pyomm(void);
 #endif
-void init_opengl(void);
-void init_opengl_num(void);
-void init_glu(void);
-void init_glu_num(void);
-void init_glut(void);
-void initopenglutil(void);
-void initopenglutil_num(void);
+
+#ifdef __cplusplus
+}
+#endif
 #endif
 #endif
 
@@ -2507,7 +2501,7 @@ void PExit(PyMOLGlobals * G, int code)
      so for the time being, let's just take the process down at this
      point, instead of allowing PyExit to be called. */
 
-  exit(EXIT_SUCCESS);
+  exit(code);
 
   Py_Exit(code);
 }
@@ -2538,7 +2532,7 @@ void PLog(PyMOLGlobals * G, char *str, int format)
   int blocked;
   PyObject *log;
   OrthoLineType buffer = "";
-  mode = (int) SettingGet(G, cSetting_logging);
+  mode = SettingGetGlobal_i(G, cSetting_logging);
   if(mode) {
     blocked = PAutoBlock(G);
     log = PyDict_GetItemString(P_pymol_dict, P_log_file_str);
@@ -2595,7 +2589,7 @@ void PLogFlush(PyMOLGlobals * G)
   int mode;
   PyObject *log;
   int blocked;
-  mode = (int) SettingGet(G, cSetting_logging);
+  mode = SettingGetGlobal_i(G, cSetting_logging);
   if(mode) {
     blocked = PAutoBlock(G);
     log = PyDict_GetItemString(P_pymol_dict, P_log_file_str);
@@ -2610,14 +2604,23 @@ int PFlush(PyMOLGlobals * G)
 {
   /* NOTE: ASSUMES unblocked Python threads and a locked API */
   PyObject *err;
-  char buffer[OrthoLineLength + 1];
   int did_work = false;
   if(OrthoCommandWaiting(G)) {
     did_work = true;
     PBlock(G);
     if(!(PIsGlutThread() && G->P_inst->glut_thread_keep_out)) {
       /* don't run if we're currently banned */
-      while(OrthoCommandOut(G, buffer)) {
+      char *buffer = 0;
+      int size, curSize = 0;
+      while(size = OrthoCommandOutSize(G)){
+	if (!curSize){
+	  buffer = VLACalloc(char, size);
+	  curSize = size;
+	} else if (size < curSize){
+	  VLASize(buffer, char, size);
+	  curSize = size;
+	}
+	OrthoCommandOut(G, buffer);
         OrthoCommandNest(G, 1);
         PUnlockAPIWhileBlocked(G);
         if(PyErr_Occurred()) {
@@ -2638,6 +2641,8 @@ int PFlush(PyMOLGlobals * G)
           PFlushFast(G);
         OrthoCommandNest(G, -1);
       }
+      if (buffer)
+	VLAFreeP(buffer);
     }
     PUnblock(G);
   }
@@ -2648,9 +2653,18 @@ int PFlushFast(PyMOLGlobals * G)
 {
   /* NOTE: ASSUMES we currently have blocked Python threads and an unlocked API */
   PyObject *err;
-  char buffer[OrthoLineLength + 1];
   int did_work = false;
-  while(OrthoCommandOut(G, buffer)) {
+  char *buffer = 0;
+  int size, curSize = 0;
+  while(size = OrthoCommandOutSize(G)){
+    if (!curSize){
+      buffer = VLACalloc(char, size);
+      curSize = size;
+    } else if (size < curSize){
+      VLASize(buffer, char, size);
+      curSize = size;
+    }
+    OrthoCommandOut(G, buffer);
     OrthoCommandNest(G, 1);
     did_work = true;
     PRINTFD(G, FB_Threads)
@@ -2674,6 +2688,9 @@ int PFlushFast(PyMOLGlobals * G)
       PFlushFast(G);
     OrthoCommandNest(G, -1);
   }
+  if (buffer)
+    VLAFreeP(buffer);
+
   return did_work;
 }
 
