@@ -80,7 +80,7 @@ void SettingUniqueSet_b(PyMOLGlobals * G, int unique_id, int setting_id, int val
 void SettingUniqueSet_i(PyMOLGlobals * G, int unique_id, int setting_id, int value);
 void SettingUniqueSet_f(PyMOLGlobals * G, int unique_id, int setting_id, float value);
 void SettingUniqueSet_color(PyMOLGlobals * G, int unique_id, int setting_id, int value);
-void SettingUniqueSetTypedValue(PyMOLGlobals * G, int unique_id, int setting_id,
+int SettingUniqueSetTypedValue(PyMOLGlobals * G, int unique_id, int setting_id,
                                 int setting_type, void *value);
 
 int SettingUniqueCheck(PyMOLGlobals * G, int unique_id, int setting_id);
@@ -109,6 +109,7 @@ void SettingCheckHandle(PyMOLGlobals * G, CSetting ** handle);
 
 int SettingSet_b(CSetting * I, int index, int value);
 int SettingSet_i(CSetting * I, int index, int value);
+int SettingSet_color_from_3f(CSetting * I, int index, float *vals);
 int SettingSet_f(CSetting * I, int index, float value);
 int SettingSet_s(CSetting * I, int index, char *value);
 int SettingSet_3f(CSetting * I, int index, float value1, float value2, float value3);
@@ -134,6 +135,7 @@ float *SettingGetGlobal_3fv(PyMOLGlobals * G, int index);       /* always succee
 
 int SettingSetGlobal_b(PyMOLGlobals * G, int index, int value);
 int SettingSetGlobal_i(PyMOLGlobals * G, int index, int value);
+int SettingSetGlobal_s(PyMOLGlobals * G, int index, char *value);
 int SettingSetGlobal_f(PyMOLGlobals * G, int index, float value);
 int SettingSetGlobal_3f(PyMOLGlobals * G, int index, float value1, float value2,
                         float value3);
@@ -169,18 +171,18 @@ int SettingStringToTypedValue(PyMOLGlobals * G, int index, char *st, int *type,
 
 #ifndef _PYMOL_NOPY
 int SettingSetFromTuple(PyMOLGlobals * G, CSetting * I, int index, PyObject * tuple);
+PyObject *SettingGetPyObject(PyMOLGlobals * G, CSetting * set1, CSetting * set2, int index);
 PyObject *SettingGetTuple(PyMOLGlobals * G, CSetting * set1, CSetting * set2, int index);       /* (type,(value,)) */
 PyObject *SettingGetDefinedTuple(PyMOLGlobals * G, CSetting * set1, int index);
 PyObject *SettingGetUpdateList(PyMOLGlobals * G, CSetting * I);
 #endif
 
-void SettingGenerateSideEffects(PyMOLGlobals * G, int index, char *sele, int state);
+void SettingGenerateSideEffects(PyMOLGlobals * G, int index, char *sele, int state, int quiet);
 
 
 /* Legacy API below */
 
 int SettingGetIndex(PyMOLGlobals * G, char *name);
-float SettingGet(PyMOLGlobals * G, int index);
 int SettingSet(PyMOLGlobals * G, int index, float v);
 int SettingSetfv(PyMOLGlobals * G, int index, float *value);
 float *SettingGetfv(PyMOLGlobals * G, int index);
@@ -195,11 +197,6 @@ int SettingSetGlobalsFromPyList(PyMOLGlobals * G, PyObject * list);
 PyObject *SettingGetGlobalsAsPyList(PyMOLGlobals * G);
 
 
-/* proposed...
-PyObject *SettingGetDefaultsAsPyList(PyMOLGlobals *G);
-int SettingSetDefaultsFromPyList(PyMOLGlobals *G,PyObject *list);
-*/
-
 CSetting *SettingNewFromPyList(PyMOLGlobals * G, PyObject * list);
 
 
@@ -289,7 +286,6 @@ CSetting *SettingNewFromPyList(PyMOLGlobals * G, PyObject * list);
 #define cSetting_selection_width              80
 #define cSetting_selection_overlay            81
 #define cSetting_static_singletons            82
-#define cSetting_max_triangles                83
 #define cSetting_depth_cue                    84
 #define cSetting_specular                     85
 #define cSetting_shininess                    86
@@ -619,6 +615,7 @@ CSetting *SettingNewFromPyList(PyMOLGlobals * G, PyObject * list);
 #define cSetting_cartoon_side_chain_helper  383
 #define cSetting_surface_optimize_subsets   384
 #define cSetting_multiplex                  385
+// cSetting_texture_fonts deprecated
 #define cSetting_texture_fonts              386
 #define cSetting_pqr_workarounds            387
 #define cSetting_animation                  388
@@ -943,6 +940,36 @@ CSetting *SettingNewFromPyList(PyMOLGlobals * G, PyObject * list);
 #define cSetting_edit_light                              707
 #define cSetting_suspend_undo                            708
 #define cSetting_suspend_undo_atom_count                 709
+#define cSetting_suspend_deferred                        710
+#define cSetting_pick_surface                            711
+// bg_image is not implemented in open-source
+#define cSetting_bg_image_filename                       712
+#define cSetting_bg_image_mode                           713
+#define cSetting_bg_image_tilesize                       714
+#define cSetting_bg_image_linear                         715
+#define cSetting_load_object_props_default               716
+#define cSetting_load_atom_props_default                 717
+#define cSetting_label_placement_offset                  718
+#define cSetting_pdb_conect_nodup                        719
+#define cSetting_label_connector                         720
+#define cSetting_label_connector_mode                    721
+#define cSetting_label_connector_color                   722
+#define cSetting_label_connector_width                   723
+#define cSetting_label_connector_ext_length              724
+#define cSetting_label_bg_color                          725
+#define cSetting_use_geometry_shaders                    726
+#define cSetting_label_relative_mode                     727
+#define cSetting_label_screen_point                      728
+#define cSetting_label_multiline_spacing                 729
+#define cSetting_label_multiline_justification           730
+#define cSetting_label_padding                           731
+#define cSetting_label_bg_transparency                   732
+#define cSetting_label_bg_outline                        733
+#define cSetting_ray_label_connector_flat                734
+#define cSetting_dash_transparency                       735
+#define cSetting_pick_labels                             736
+#define cSetting_label_z_target                          737
+#define cSetting_session_embeds_data                     738
 
 /* when you add a new setting also remember:
    layer1/Setting.c
@@ -953,6 +980,6 @@ CSetting *SettingNewFromPyList(PyMOLGlobals * G, PyObject * list);
 
 /* cSetting_ss_INIT must always be last setting_index +1 */
 
-#define cSetting_INIT                       710
+#define cSetting_INIT                       739
 
 #endif
