@@ -718,133 +718,91 @@ static int WizardRelease(Block * block, int button, int x, int y, int mod)
 }
 
 static void draw_button(int x2, int y2, int w, int h, float *light, float *dark,
-                        float *inside)
+                        float *inside ORTHOCGOARG)
 {
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-  glColor3fv(light);
-#ifdef _PYMOL_GL_DRAWARRAYS
-  {
-    const GLint polyVerts[] = {
-      x2, y2,
-      x2, y2 + h,
-      x2 + w, y2,
-      x2 + w, y2 + h
-    };
-    glEnableClientState(GL_VERTEX_ARRAY);
-    glVertexPointer(2, GL_INT, 0, polyVerts);
-    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-    glDisableClientState(GL_VERTEX_ARRAY);
-  }
-#else
-  glBegin(GL_POLYGON);
-  glVertex2i(x2, y2);
-  glVertex2i(x2, y2 + h);
-  glVertex2i(x2 + w, y2 + h);
-  glVertex2i(x2 + w, y2);
-  glEnd();
-#endif
-#endif
+    if (orthoCGO){
+      CGOColorv(orthoCGO, light);
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, x2, y2, 0.f);
+      CGOVertex(orthoCGO, x2, y2 + h, 0.f);
+      CGOVertex(orthoCGO, x2 + w, y2, 0.f);
+      CGOVertex(orthoCGO, x2 + w, y2 + h, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glColor3fv(light);
+      glBegin(GL_POLYGON);
+      glVertex2i(x2, y2);
+      glVertex2i(x2, y2 + h);
+      glVertex2i(x2 + w, y2 + h);
+      glVertex2i(x2 + w, y2);
+      glEnd();
+    }
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-  glColor3fv(dark);
-#ifdef _PYMOL_GL_DRAWARRAYS
-  {
-    const GLint polyVerts[] = {
-      x2 + 1, y2,
-      x2 + 1, y2 + h - 1,
-      x2 + w, y2,
-      x2 + w, y2 + h - 1
-    };
-    glEnableClientState(GL_VERTEX_ARRAY);
-    glVertexPointer(2, GL_INT, 0, polyVerts);
-    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-    glDisableClientState(GL_VERTEX_ARRAY);
-  }
-#else
-  glBegin(GL_POLYGON);
-  glVertex2i(x2 + 1, y2);
-  glVertex2i(x2 + 1, y2 + h - 1);
-  glVertex2i(x2 + w, y2 + h - 1);
-  glVertex2i(x2 + w, y2);
-  glEnd();
-#endif
-#endif
+    if (orthoCGO){
+      CGOColorv(orthoCGO, dark);
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, x2 + 1, y2, 0.f);
+      CGOVertex(orthoCGO, x2 + 1, y2 + h - 1, 0.f);
+      CGOVertex(orthoCGO, x2 + w, y2, 0.f);
+      CGOVertex(orthoCGO, x2 + w, y2 + h - 1, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glColor3fv(dark);
+      glBegin(GL_POLYGON);
+      glVertex2i(x2 + 1, y2);
+      glVertex2i(x2 + 1, y2 + h - 1);
+      glVertex2i(x2 + w, y2 + h - 1);
+      glVertex2i(x2 + w, y2);
+      glEnd();
+    }
 
   if(inside) {
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3fv(inside);
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	x2 + 1, y2 + 1,
-	x2 + 1, y2 + h - 1,
-	x2 + w - 1, y2 + 1,
-	x2 + w - 1, y2 + h - 1
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
+    if (orthoCGO){
+      CGOColorv(orthoCGO, inside);
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, x2 + 1, y2 + 1, 0.f);
+      CGOVertex(orthoCGO, x2 + 1, y2 + h - 1, 0.f);
+      CGOVertex(orthoCGO, x2 + w - 1, y2 + 1, 0.f);
+      CGOVertex(orthoCGO, x2 + w - 1, y2 + h - 1, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glColor3fv(inside);
+      glBegin(GL_POLYGON);
+      glVertex2i(x2 + 1, y2 + 1);
+      glVertex2i(x2 + 1, y2 + h - 1);
+      glVertex2i(x2 + w - 1, y2 + h - 1);
+      glVertex2i(x2 + w - 1, y2 + 1);
+      glEnd();
     }
-#else
-    glBegin(GL_POLYGON);
-    glVertex2i(x2 + 1, y2 + 1);
-    glVertex2i(x2 + 1, y2 + h - 1);
-    glVertex2i(x2 + w - 1, y2 + h - 1);
-    glVertex2i(x2 + w - 1, y2 + 1);
-    glEnd();
-#endif
-#endif
   } else {                      /* rainbow */
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	x2 + 1, y2 + 1,
-	x2 + 1, y2 + h - 1,
-	x2 + w - 1, y2 + 1,
-	x2 + w - 1, y2 + h - 1
-      };
-      const GLfloat colorVals[] = {
-	1.0F, 0.1F, 0.1F, 1.f,
-	0.1F, 1.0F, 0.1F, 1.f,
-	0.1F, 0.1F, 1.0F, 1.f,
-	1.0F, 1.0F, 0.1F, 1.f
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glEnableClientState(GL_COLOR_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glColorPointer(4, GL_FLOAT, 0, colorVals);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
-      glDisableClientState(GL_COLOR_ARRAY);
+    if (orthoCGO){
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOColor(orthoCGO, 0.1F, 1.0F, 0.1F); // green
+      CGOVertex(orthoCGO, x2 + 1, y2 + h - 1, 0.f);
+      CGOColor(orthoCGO, 1.0F, 1.0F, 0.1F);  // yellow
+      CGOVertex(orthoCGO, x2 + w - 1, y2 + h - 1, 0.f);
+      CGOColor(orthoCGO, 1.f, 0.1f, 0.1f); // red
+      CGOVertex(orthoCGO, x2 + 1, y2 + 1, 0.f);
+      CGOColor(orthoCGO, 0.1F, 0.1F, 1.0F);  // blue
+      CGOVertex(orthoCGO, x2 + w - 1, y2 + 1, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glBegin(GL_POLYGON);
+      glColor3f(1.0F, 0.1F, 0.1F);
+      glVertex2i(x2 + 1, y2 + 1);
+      glColor3f(0.1F, 1.0F, 0.1F);
+      glVertex2i(x2 + 1, y2 + h - 1);
+      glColor3f(1.0F, 1.0F, 0.1F);
+      glVertex2i(x2 + w - 1, y2 + h - 1);
+      glColor3f(0.1F, 0.1F, 1.0F);
+      glVertex2i(x2 + w - 1, y2 + 1);
+      glEnd();
     }
-#else
-    glBegin(GL_POLYGON);
-    glColor3f(1.0F, 0.1F, 0.1F);
-    glVertex2i(x2 + 1, y2 + 1);
-    glColor3f(0.1F, 1.0F, 0.1F);
-    glVertex2i(x2 + 1, y2 + h - 1);
-    glColor3f(1.0F, 1.0F, 0.1F);
-    glVertex2i(x2 + w - 1, y2 + h - 1);
-    glColor3f(0.1F, 0.1F, 1.0F);
-    glVertex2i(x2 + w - 1, y2 + 1);
-    glEnd();
-#endif
-#endif
   }
 
 }
 
-static void draw_text(PyMOLGlobals * G, char *c, int xx, int yy, float *color)
+static void draw_text(PyMOLGlobals * G, char *c, int xx, int yy, float *color ORTHOCGOARG)
 {
   TextSetColor(G, color);
   while(*c) {
@@ -862,14 +820,14 @@ static void draw_text(PyMOLGlobals * G, char *c, int xx, int yy, float *color)
             }
           }
     TextSetPos2i(G, xx, yy);
-    TextDrawChar(G, *(c++));
+    TextDrawChar(G, *(c++) ORTHOCGOARGVAR);
     xx = xx + 8;
   }
 }
 
 
 /*========================================================================*/
-static void WizardDraw(Block * block)
+static void WizardDraw(Block * block ORTHOCGOARG)
 {
   PyMOLGlobals *G = block->G;
 
@@ -900,29 +858,26 @@ static void WizardDraw(Block * block)
   if(G->HaveGUI && G->ValidContext && ((block->rect.right - block->rect.left) > 6)) {
 
     if(SettingGetGlobal_b(G, cSetting_internal_gui_mode) == 0) {
-#ifdef PURE_OPENGL_ES_2
-		/* TODO */
-#else
+    if (orthoCGO)
+      CGOColorv(orthoCGO, I->Block->BackColor);
+    else
       glColor3fv(I->Block->BackColor);
-#endif
-      BlockFill(I->Block);
-      BlockDrawLeftEdge(I->Block);
+      BlockFill(I->Block ORTHOCGOARGVAR);
+      BlockDrawLeftEdge(I->Block ORTHOCGOARGVAR);
     } else {
-      BlockDrawLeftEdge(I->Block);
-#ifdef PURE_OPENGL_ES_2
-		/* TODO */
-#else
+      BlockDrawLeftEdge(I->Block ORTHOCGOARGVAR);
+    if (orthoCGO)
+      CGOColor(orthoCGO, .5f, .5f, .5f);
+    else
       glColor3f(0.5, 0.5, 0.5);
-#endif
       BlockDrawTopEdge(I->Block);
       text_color2 = OrthoGetOverlayColor(G);
     }
 
-#ifdef PURE_OPENGL_ES_2
-		/* TODO */
-#else
-    glColor3fv(I->Block->TextColor);
-#endif
+    if (orthoCGO)
+      CGOColorv(orthoCGO, I->Block->TextColor);
+    else
+      glColor3fv(I->Block->TextColor);
     x = I->Block->rect.left + cWizardLeftMargin;
     y = (I->Block->rect.top - LineHeight) - cWizardTopMargin;
 
@@ -930,23 +885,19 @@ static void WizardDraw(Block * block)
       if(I->Pressed == a) {
         draw_button(I->Block->rect.left + 1, y,
                     (I->Block->rect.right - I->Block->rect.left) - 1,
-                    LineHeight - 1, dimLightEdge, dimDarkEdge, buttonActiveColor);
+                    LineHeight - 1, dimLightEdge, dimDarkEdge, buttonActiveColor ORTHOCGOARGVAR);
         /*        glColor3f(0.0,0.0,0.0); */
         text_color = black_color;
       } else {
         switch (I->Line[a].type) {
         case cWizTypeText:
           text_color = text_color2;
-#ifdef PURE_OPENGL_ES_2
-		/* TODO */
-#else
           glColor3fv(text_color2);
-#endif
           break;
         case cWizTypeButton:
           draw_button(I->Block->rect.left + 1, y,
                       (I->Block->rect.right - I->Block->rect.left) - 1,
-                      LineHeight - 1, dimLightEdge, dimDarkEdge, dimColor);
+                      LineHeight - 1, dimLightEdge, dimDarkEdge, dimColor ORTHOCGOARGVAR);
 
           /*          glColor3fv(buttonTextColor); */
           text_color = buttonTextColor;
@@ -954,7 +905,7 @@ static void WizardDraw(Block * block)
         case cWizTypePopUp:
           draw_button(I->Block->rect.left + 1, y,
                       (I->Block->rect.right - I->Block->rect.left) - 1,
-                      LineHeight - 1, menuLightEdge, menuDarkEdge, menuBGColor);
+                      LineHeight - 1, menuLightEdge, menuDarkEdge, menuBGColor ORTHOCGOARGVAR);
           /* glColor3fv(menuColor); */
           text_color = menuColor;
           break;
@@ -962,7 +913,7 @@ static void WizardDraw(Block * block)
           break;
         }
       }
-      draw_text(G, I->Line[a].text, x + 1, y + text_lift, text_color);
+      draw_text(G, I->Line[a].text, x + 1, y + text_lift, text_color ORTHOCGOARGVAR);
       /*GrapDrawStr(I->Line[a].text,x+1,y+text_lift); */
       y -= LineHeight;
     }
