@@ -931,26 +931,6 @@ int AtomInfoKnownPolymerResName(char *resn)
 
 /*========================================================================*/
 
-static int AtomInfoInOrder(PyMOLGlobals * G, AtomInfoType * atom, int atom1, int atom2)
-{
-  return (AtomInfoCompare(G, atom + atom1, atom + atom2) <= 0);
-}
-
-static int AtomInfoInOrderIgnoreHet(PyMOLGlobals * G, AtomInfoType * atom, int atom1,
-                                    int atom2)
-{
-  return (AtomInfoCompareIgnoreHet(G, atom + atom1, atom + atom2) <= 0);
-}
-
-static int AtomInfoInOrigOrder(PyMOLGlobals * G, AtomInfoType * atom, int atom1,
-                               int atom2)
-{
-  if(atom[atom1].rank != atom[atom2].rank)
-    return (atom[atom1].rank < atom[atom2].rank);
-  else
-    return (AtomInfoCompare(G, atom + atom1, atom + atom2) <= 0);
-}
-
 int AtomResvFromResi(char *resi)
 {
   int result = 1;
@@ -1500,7 +1480,7 @@ void AtomInfoBracketResidueFast(PyMOLGlobals * G, AtomInfoType * ai0, int n0, in
 int AtomInfoUpdateAutoColor(PyMOLGlobals * G)
 {
   CAtomInfo *I = G->AtomInfo;
-  if(SettingGet(G, cSetting_auto_color))
+  if(SettingGetGlobal_b(G, cSetting_auto_color))
     I->CColor = ColorGetNext(G);
   else
     I->CColor = ColorGetIndex(G, "carbon");
@@ -2392,33 +2372,6 @@ int AtomInfoGetColorWithElement(PyMOLGlobals * G, AtomInfoType * at1, char *n)
   return (color);
 }
 
-int *AtomInfoGetSortedIndex(PyMOLGlobals * G, CObject * obj, AtomInfoType * rec, int n,
-                            int **outdex)
-{
-  int *index;
-  int a;
-  CSetting *setting = NULL;
-  index = Alloc(int, n + 1);
-  ErrChkPtr(G, index);
-  (*outdex) = Alloc(int, n + 1);
-  ErrChkPtr(G, *outdex);
-  if(obj)
-    setting = obj->Setting;
-
-  if(SettingGet_b(G, setting, NULL, cSetting_retain_order)) {
-    UtilSortIndexGlobals(G, n, rec, index, (UtilOrderFnGlobals *) AtomInfoInOrigOrder);
-  } else if(SettingGet_b(G, setting, NULL, cSetting_pdb_hetatm_sort)) {
-    UtilSortIndexGlobals(G, n, rec, index, (UtilOrderFnGlobals *) AtomInfoInOrder);
-  } else {
-    UtilSortIndexGlobals(G, n, rec, index,
-                         (UtilOrderFnGlobals *) AtomInfoInOrderIgnoreHet);
-  }
-
-  for(a = 0; a < n; a++)
-    (*outdex)[index[a]] = a;
-  return (index);
-}
-
 void AtomInfoFreeSortedIndexes(PyMOLGlobals * G, int **index, int **outdex)
 {
   FreeP(*index);
@@ -3511,7 +3464,7 @@ void AtomInfoAssignParameters(PyMOLGlobals * G, AtomInfoType * I)
     n++;
   if(toupper(*n) != I->elem[0]) {
     pri = 1000;                 /* unconventional atom name -- make no assignments */
-  } else if((int) SettingGet(G, cSetting_pdb_standard_order)) {
+  } else if(SettingGetGlobal_b(G, cSetting_pdb_standard_order)) {
     switch (*n) {
 
     case 'N':
@@ -3958,7 +3911,7 @@ void AtomInfoAssignParameters(PyMOLGlobals * G, AtomInfoType * I)
     break;                      /* default radius for known atoms with unknown radii */
   }
 
-  if(SettingGet(G, cSetting_legacy_vdw_radii)) {        /* ver<0.75, old, incorrect VDW */
+  if(SettingGetGlobal_b(G, cSetting_legacy_vdw_radii)) {        /* ver<0.75, old, incorrect VDW */
     if(!strcmp(e, "N"))
       vdw = 1.8F;               /* slow but compact */
     if(!strcmp(e, "C"))
@@ -3984,11 +3937,6 @@ void AtomInfoAssignParameters(PyMOLGlobals * G, AtomInfoType * I)
   if(I->vdw == 0.0)             /* only assigned if not yet assigned */
     I->vdw = vdw;
 
-#if 0
-  if(!I->protons)
-    I->protons = cAN_C;         /* default assumption */
-#endif
-
   if(I->protons == cAN_H)
     I->hydrogen = true;
   /*  printf("I->name %s I->priority %d\n",I->name,I->priority); */
