@@ -33,23 +33,19 @@ Z* -------------------------------------------------------------------
 #include"VFont.h"
 #include"ShaderMgr.h"
 
-static void ObjectCGOFree(ObjectCGO * I);
-
 #ifndef _PYMOL_NOPY
 static PyObject *ObjectCGOStateAsPyList(ObjectCGOState * I)
 {
 
   PyObject *result = NULL;
 
-  result = PyList_New(2);
-  if(I->std)
+  result = PyList_New(1);
+  if(I->ray)
+    PyList_SetItem(result, 0, CGOAsPyList(I->ray));
+  else if(I->std)
     PyList_SetItem(result, 0, CGOAsPyList(I->std));
   else
     PyList_SetItem(result, 0, PConvAutoNone(NULL));
-  if(I->ray)
-    PyList_SetItem(result, 1, CGOAsPyList(I->ray));
-  else
-    PyList_SetItem(result, 1, PConvAutoNone(NULL));
   return (PConvAutoNone(result));
 
 }
@@ -64,14 +60,13 @@ static PyObject *ObjectCGOAllStatesAsPyList(ObjectCGO * I)
     PyList_SetItem(result, a, ObjectCGOStateAsPyList(I->State + a));
   }
   return (PConvAutoNone(result));
-
 }
 
 static int ObjectCGOStateFromPyList(PyMOLGlobals * G, ObjectCGOState * I, PyObject * list,
                                     int version)
 {
   int ok = true;
-  int ll;
+  int ll, pl = 0;
   PyObject *tmp;
   if(ok)
     ok = (list != NULL);
@@ -81,19 +76,24 @@ static int ObjectCGOStateFromPyList(PyMOLGlobals * G, ObjectCGOState * I, PyObje
     ll = PyList_Size(list);
   /* TO SUPPORT BACKWARDS COMPATIBILITY...
      Always check ll when adding new PyList_GetItem's */
-  if(ok) {
+
+  if(ok && ll==2) {
     tmp = PyList_GetItem(list, 0);
     if(tmp == Py_None)
       I->std = NULL;
     else
-      ok = ((I->std = CGONewFromPyList(G, PyList_GetItem(list, 0), version)) != NULL);
+      ok = ((I->std = CGONewFromPyList(G, tmp, version)) != NULL);
+    pl++;
   }
   if(ok) {
-    tmp = PyList_GetItem(list, 1);
+    tmp = PyList_GetItem(list, pl);
     if(tmp == Py_None)
       I->ray = NULL;
     else
-      ok = ((I->ray = CGONewFromPyList(G, PyList_GetItem(list, 1), version)) != NULL);
+      ok = ((I->ray = CGONewFromPyList(G, tmp, version)) != NULL);
+    if (!I->std && I->ray){
+      I->std = CGOSimplify(I->ray, 0);
+    }
   }
   return (ok);
 }
@@ -174,7 +174,7 @@ PyObject *ObjectCGOAsPyList(ObjectCGO * I)
 
 /*========================================================================*/
 
-static void ObjectCGOFree(ObjectCGO * I)
+void ObjectCGOFree(ObjectCGO * I)
 {
   int a;
   for(a = 0; a < I->NState; a++) {
@@ -230,14 +230,25 @@ void ObjectCGORecomputeExtent(ObjectCGO * I)
 /*========================================================================*/
 static void ObjectCGOInvalidate(ObjectCGO * I, int rep, int level, int state)
 {
+  ObjectCGOState *sobj = NULL;
   if(state < 0) {
     int a;
     for(a = 0; a < I->NState; a++) {
       I->State[a].valid = false;
+      sobj = I->State + a;
+      if (sobj->shaderCGO){
+	CGOFree(sobj->shaderCGO);	      
+	sobj->shaderCGO = 0;
+      }
     }
   } else {
     if((state >= 0) && (state < I->NState)) {
       I->State[state].valid = false;
+      sobj = I->State + state;
+      if (sobj->shaderCGO){
+	CGOFree(sobj->shaderCGO);	      
+	sobj->shaderCGO = 0;
+      }
     }
   }
 }
@@ -261,16 +272,6 @@ static void ObjectCGOUpdate(ObjectCGO * I)
           if(ocs->std)
             CGOFree(ocs->std);
           ocs->std = CGOSimplify(ocs->ray, est);
-#ifdef _PYMOL_CGO_DRAWARRAYS
-	  {
-	    CGO *convertcgo = NULL;
-	    if(ocs->std && ocs->std->has_begin_end){
-	      convertcgo = CGOCombineBeginEnd(ocs->std, 0);
-	      CGOFree(ocs->std);
-	      ocs->std = convertcgo;
-	    }
-	  }
-#endif
         }
       }
       ocs->valid = true;
@@ -302,9 +303,9 @@ static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
   float *color;
   int use_shader = 0;
   
-  use_shader = (int) SettingGet(G, cSetting_cgo_use_shader) &
-    (int)SettingGet(G, cSetting_use_shaders) & 
-    (int)!SettingGet(G, cSetting_transparency_global_sort);
+  use_shader = SettingGetGlobal_b(G, cSetting_cgo_use_shader) &
+    SettingGetGlobal_b(G, cSetting_use_shaders) & 
+    !SettingGetGlobal_b(G, cSetting_transparency_global_sort);
 
   ObjectPrepareContext(&I->Obj, ray);
 
@@ -331,7 +332,20 @@ static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
 		  colorWithA[0] = 1.f; colorWithA[1] = 1.f; colorWithA[2] = 1.f;
 		}
 		colorWithA[3] = 1.f - SettingGet_f(G, I->Obj.Setting, NULL, cSetting_cgo_transparency);
-		convertcgo = CGOOptimizeToVBOIndexedWithColor(sobj->std, 0, colorWithA);
+		{
+		  CGO *convertcgo = NULL;
+		  if(sobj->std && sobj->std->has_begin_end){
+		    convertcgo = CGOCombineBeginEnd(sobj->std, 0);
+		    CGOFree(sobj->std);
+		    sobj->std = convertcgo;
+		  }
+		}
+		if (CGOHasCylinderOperations(sobj->std)){
+		  convertcgo = CGOOptimizeGLSLCylindersToVBOIndexedNoColor(sobj->std, 0);
+		  //		  convertcgo->enable_shaders = true;
+		} else {
+		  convertcgo = CGOOptimizeToVBOIndexedWithColor(sobj->std, 0, colorWithA);
+		}
 		sobj->shaderCGO = convertcgo;
 	      }
 	    } else if (sobj->shaderCGO){
@@ -340,10 +354,24 @@ static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
 	    }
 #endif
             if(ray) {
-              if(sobj->ray)
-                CGORenderRay(sobj->ray, ray, color, I->Obj.Setting, NULL);
-              else
-                CGORenderRay(sobj->std, ray, color, I->Obj.Setting, NULL);
+	      int try_std = false;
+              if(sobj->ray){
+                int rayok = CGORenderRay(sobj->ray, ray, color, I->Obj.Setting, NULL);
+		if (!rayok){
+		  CGOFree(sobj->ray);
+		  sobj->ray = NULL;
+		  try_std = true;
+		}
+	      } else {
+		try_std = true;
+	      }
+	      if (try_std && sobj->std){
+		int rayok = CGORenderRay(sobj->std, ray, color, I->Obj.Setting, NULL);
+		if (!rayok){
+		  CGOFree(sobj->std);
+		  sobj->std = NULL;
+		}
+	      }
             } else if(G->HaveGUI && G->ValidContext) {
               if(pick) {
               } else {
@@ -356,17 +384,15 @@ static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
 		}
 		if (use_shader && sobj->shaderCGO){
 		  shaderPrg = CShaderPrg_Enable_DefaultShader(G);
-		  CShaderPrg_Set1i(shaderPrg, "lighting_enabled", cgo_lighting);
+		  CShaderPrg_SetLightingEnabled(shaderPrg, cgo_lighting);
 		  CShaderPrg_Set1i(shaderPrg, "two_sided_lighting_enabled", two_sided_lighting);
 		  sobj->shaderCGO->use_shader = use_shader;
-		  sobj->shaderCGO->debug = SettingGet(G, cSetting_cgo_debug);
+		  sobj->shaderCGO->debug = SettingGetGlobal_i(G, cSetting_cgo_debug);
 		  CGORenderGL(sobj->shaderCGO, color, I->Obj.Setting, NULL, info, NULL);
 		  CShaderPrg_Disable(shaderPrg);
 		} else {
 		  sobj->std->use_shader = use_shader;
-		  sobj->std->debug = SettingGet(G, cSetting_cgo_debug);
-#ifdef PURE_OPENGL_ES_2
-#else
+		  sobj->std->debug = SettingGetGlobal_i(G, cSetting_cgo_debug);
 		  if (cgo_lighting){
 		    glEnable(GL_LIGHTING);
 		  } else {
@@ -377,17 +403,14 @@ static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
 		  } else {
 		    GLLIGHTMODELI(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
 		  }
-#endif
 		  sobj->std->use_shader = use_shader;
-		  sobj->std->debug = SettingGet(G, cSetting_cgo_debug);
+		  sobj->std->debug = SettingGetGlobal_i(G, cSetting_cgo_debug);
 		  CGORenderGL(sobj->std, color, I->Obj.Setting, NULL, info, NULL);
-#ifndef _PYMOL_PURE_OPENGL_ES
 		  if (SceneGetTwoSidedLighting(G)){
 		    glEnable(GL_VERTEX_PROGRAM_TWO_SIDE);
 		  } else {
 		    glDisable(GL_VERTEX_PROGRAM_TWO_SIDE);
 		  }
-#endif
           if (!cgo_lighting){
 		    glEnable(GL_LIGHTING);
 		  }
@@ -398,7 +421,7 @@ static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
         }
       } else {
         if(!sobj) {
-          if(I->NState && SettingGet(G, cSetting_static_singletons))
+          if(I->NState && SettingGetGlobal_b(G, cSetting_static_singletons))
             sobj = I->State;
         }
 #ifdef _PYMOL_CGO_DRAWBUFFERS
@@ -411,7 +434,20 @@ static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
 	      colorWithA[0] = 1.f; colorWithA[1] = 1.f; colorWithA[2] = 1.f;
 	    }
 	    colorWithA[3] = 1.f - SettingGet_f(G, I->Obj.Setting, NULL, cSetting_cgo_transparency);
-	    sobj->shaderCGO = CGOOptimizeToVBOIndexedWithColor(sobj->std, 0, colorWithA);
+	    {
+	      CGO *convertcgo = NULL;
+	      if(sobj->std && sobj->std->has_begin_end){
+		convertcgo = CGOCombineBeginEnd(sobj->std, 0);
+		CGOFree(sobj->std);
+		sobj->std = convertcgo;
+	      }
+	    }
+	    if (CGOHasCylinderOperations(sobj->std)){
+	      sobj->shaderCGO = CGOOptimizeGLSLCylindersToVBOIndexedNoColor(sobj->std, 0);
+	      //	      sobj->shaderCGO->enable_shaders = true;
+	    } else {
+	      sobj->shaderCGO = CGOOptimizeToVBOIndexedWithColor(sobj->std, 0, colorWithA);
+	    }
 	  }
 	} else if (sobj->shaderCGO){
 	  CGOFree(sobj->shaderCGO);	      
@@ -420,10 +456,24 @@ static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
 #endif
         if(ray) {
           if(sobj) {
-            if(sobj->ray)
-              CGORenderRay(sobj->ray, ray, color, I->Obj.Setting, NULL);
-            else if(sobj->std)
-              CGORenderRay(sobj->std, ray, color, I->Obj.Setting, NULL);
+	    int try_std = false;
+            if(sobj->ray){
+              int rayok = CGORenderRay(sobj->ray, ray, color, I->Obj.Setting, NULL);
+	      if (!rayok){
+		CGOFree(sobj->ray);
+		sobj->ray = NULL;
+		try_std = true;
+	      }
+	    } else {
+	      try_std = true;
+	    }
+	    if (try_std && sobj->std){
+              int rayok = CGORenderRay(sobj->std, ray, color, I->Obj.Setting, NULL);
+	      if (!rayok){
+		CGOFree(sobj->std);
+		sobj->std = NULL;
+	      }
+	    }
           }
         } else if(G->HaveGUI && G->ValidContext) {
           if(pick) {
@@ -439,35 +489,32 @@ static void ObjectCGORender(ObjectCGO * I, RenderInfo * info)
 		}
 		if (use_shader){
 		  shaderPrg = CShaderPrg_Enable_DefaultShader(G);
-		  CShaderPrg_Set1i(shaderPrg, "lighting_enabled", cgo_lighting);
+		  if (!shaderPrg) return;
+		  CShaderPrg_SetLightingEnabled(shaderPrg, cgo_lighting);
 		  CShaderPrg_Set1i(shaderPrg, "two_sided_lighting_enabled", two_sided_lighting);
 		  sobj->shaderCGO->use_shader = use_shader;
-		  sobj->shaderCGO->debug = SettingGet(G, cSetting_cgo_debug);
+		  sobj->shaderCGO->debug = SettingGetGlobal_i(G, cSetting_cgo_debug);
 		  CGORenderGL(sobj->shaderCGO, color, I->Obj.Setting, NULL, info, NULL);
 		  CShaderPrg_Disable(shaderPrg);
 		} else {
 		  sobj->std->use_shader = use_shader;
-		  sobj->std->debug = SettingGet(G, cSetting_cgo_debug);
+		  sobj->std->debug = SettingGetGlobal_i(G, cSetting_cgo_debug);
 		  if (cgo_lighting){
 		    glEnable(GL_LIGHTING);
 		  } else {
 		    glDisable(GL_LIGHTING);
 		  }
-#ifndef _PYMOL_PURE_OPENGL_ES
 		  if (two_sided_lighting){
 		    glEnable(GL_VERTEX_PROGRAM_TWO_SIDE);
 		  } else {
 		    glDisable(GL_VERTEX_PROGRAM_TWO_SIDE);
 		  }
-#endif
 		  CGORenderGL(sobj->std, color, I->Obj.Setting, NULL, info, NULL);
-#ifndef _PYMOL_PURE_OPENGL_ES
 		  if (SceneGetTwoSidedLighting(G)){
 		    glEnable(GL_VERTEX_PROGRAM_TWO_SIDE);
 		  } else {
 		    glDisable(GL_VERTEX_PROGRAM_TWO_SIDE);
 		  }
-#endif
 		  if (!cgo_lighting){
 		    glEnable(GL_LIGHTING);
 		  }
@@ -488,7 +535,7 @@ ObjectCGO *ObjectCGONew(PyMOLGlobals * G)
 
   ObjectInit(G, (CObject *) I);
 
-  I->State = VLAMalloc(10, sizeof(ObjectCGOState), 5, true);
+  I->State = VLACalloc(ObjectCGOState, 10);
   I->NState = 0;
   I->Obj.type = cObjectCGO;
   I->Obj.fFree = (void (*)(CObject *)) ObjectCGOFree;
@@ -524,16 +571,6 @@ static CGO *ObjectCGOPyListFloatToCGO(PyMOLGlobals * G, PyObject * list)
             PRINTF " FloatToCGO: error encountered on element %d\n", result ENDF(G);
           }
           CGOStop(cgo);
-#ifdef _PYMOL_CGO_DRAWARRAYS
-	  {
-	    CGO *convertcgo = NULL;
-	    if(cgo && cgo->has_begin_end){
-	      convertcgo = CGOCombineBeginEnd(cgo, 0);
-	      CGOFree(cgo);
-	      cgo = convertcgo;
-	    }
-	  }
-#endif
         }
       }
       FreeP(raw);
@@ -571,7 +608,7 @@ static CGO *ObjectCGOFloatArrayToCGO(PyMOLGlobals * G, float *raw, int len, int
 ObjectCGO *ObjectCGOFromCGO(PyMOLGlobals * G, ObjectCGO * obj, CGO * cgo, int state)
 {
   ObjectCGO *I = NULL;
-  int est;
+  int est = 0;
 
   if(obj) {
     if(obj->Obj.type != cObjectCGO)     /* TODO: handle this */
@@ -599,17 +636,8 @@ ObjectCGO *ObjectCGOFromCGO(PyMOLGlobals * G, ObjectCGO * obj, CGO * cgo, int st
   if(I->State[state].ray) {
     CGOFree(I->State[state].ray);
   }
-  est = CGOCheckComplex(cgo);
-#ifdef _PYMOL_CGO_DRAWARRAYS
-  {
-    CGO *convertcgo = NULL;
-    if(cgo && cgo->has_begin_end){
-      convertcgo = CGOCombineBeginEnd(cgo, 0);
-      CGOFree(cgo);
-      cgo = convertcgo;
-    }
-  }
-#endif
+  if (cgo)
+    est = CGOCheckComplex(cgo);
   if(est) {
     I->State[state].ray = cgo;
     I->State[state].std = CGOSimplify(cgo, est);
@@ -694,16 +722,6 @@ ObjectCGO *ObjectCGODefine(PyMOLGlobals * G, ObjectCGO * obj, PyObject * pycgo,
             cgo = font_cgo;
           }
           est = CGOCheckComplex(cgo);
-#ifdef _PYMOL_CGO_DRAWARRAYS
-	  {
-	    CGO *convertcgo = NULL;
-	    if(cgo && cgo->has_begin_end){
-	      convertcgo = CGOCombineBeginEnd(cgo, 0);
-	      CGOFree(cgo);
-	      cgo = convertcgo;
-	    }
-	  }
-#endif
           if(est) {
             I->State[state].ray = cgo;
             I->State[state].std = CGOSimplify(cgo, est);
@@ -769,16 +787,6 @@ ObjectCGO *ObjectCGOFromFloatArray(PyMOLGlobals * G, ObjectCGO * obj,
       cgo = font_cgo;
     }
     est = CGOCheckComplex(cgo);
-#ifdef _PYMOL_CGO_DRAWARRAYS
-    {
-      CGO *convertcgo = NULL;
-      if(cgo && cgo->has_begin_end){
-	convertcgo = CGOCombineBeginEnd(cgo, 0);
-	CGOFree(cgo);
-	cgo = convertcgo;
-      }
-    }
-#endif
     if(est) {
       I->State[state].ray = cgo;
       I->State[state].std = CGOSimplify(cgo, est);
