@@ -39,6 +39,7 @@ Z* -------------------------------------------------------------------
 #include"Matrix.h"
 #include"ShaderMgr.h"
 #include"CGO.h"
+#include"ObjectCGO.h"
 
 ObjectMesh *ObjectMeshNew(PyMOLGlobals * G);
 
@@ -84,6 +85,23 @@ static PyObject *ObjectMeshStateAsPyList(ObjectMeshState * I)
 #endif
 
 #ifndef _PYMOL_NOPY
+static int ObjectMeshStateMapExists(ObjectMesh *I, ObjectMeshState *ms){
+  return ExecutiveFindObjectMapByName(I->Obj.G, ms->MapName) ? 1 : 0;
+}
+
+int ObjectMeshAllMapsInStatesExist(ObjectMesh * I)
+{
+  int a;
+  for(a = 0; a < I->NState; a++) {
+    if(I->State[a].Active) {
+      if (!ObjectMeshStateMapExists(I, I->State + a)){
+	return 0;
+      }
+    }
+  }
+  return 1;
+}
+
 static PyObject *ObjectMeshAllStatesAsPyList(ObjectMesh * I)
 {
 
@@ -237,12 +255,15 @@ int ObjectMeshNewFromPyList(PyMOLGlobals * G, PyObject * list, ObjectMesh ** res
     (*result) = I;
     ObjectMeshRecomputeExtent(I);
   } else {
-    /* cleanup? */
+    ObjectMeshFree(I);
+    (*result) = NULL;
   }
   return (ok);
 #endif
 }
 
+static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCGO, int stateArg);
+
 PyObject *ObjectMeshAsPyList(ObjectMesh * I)
 {
 #ifdef _PYMOL_NOPY
@@ -251,11 +272,32 @@ PyObject *ObjectMeshAsPyList(ObjectMesh * I)
 
   PyObject *result = NULL;
 
-  result = PyList_New(3);
-  PyList_SetItem(result, 0, ObjectAsPyList(&I->Obj));
-  PyList_SetItem(result, 1, PyInt_FromLong(I->NState));
-  PyList_SetItem(result, 2, ObjectMeshAllStatesAsPyList(I));
+  int allMapsExist = ObjectMeshAllMapsInStatesExist(I);
 
+  if (allMapsExist){
+    result = PyList_New(3);
+    PyList_SetItem(result, 0, ObjectAsPyList(&I->Obj));
+    PyList_SetItem(result, 1, PyInt_FromLong(I->NState));
+    PyList_SetItem(result, 2, ObjectMeshAllStatesAsPyList(I));
+  } else {
+    /* save ObjectMesh as ObjectCGO */
+    ObjectCGO *retObjectCGO = NULL;
+    int a;
+    PRINTFB(I->Obj.G, FB_ObjectMesh, FB_Errors)
+      "ObjectMesh-Warning: map has been deleted, saving as CGO.\n"
+      ENDFB(I->Obj.G);
+    for(a = 0; a < I->NState; a++) {
+      CGO *cgo = ObjectMeshRenderImpl(I, 0, 1, a);
+      retObjectCGO = ObjectCGOFromCGO(I->Obj.G, retObjectCGO, cgo, a);
+      
+    }
+    ObjectCopyHeader(&retObjectCGO->Obj, &I->Obj);
+    ObjectSetRepVis(&retObjectCGO->Obj, cRepMesh, 0);
+    ObjectSetRepVis(&retObjectCGO->Obj, cRepCGO, 1);
+    retObjectCGO->Obj.type = cObjectCGO;
+    result = ObjectCGOAsPyList(retObjectCGO);
+    ObjectCGOFree(retObjectCGO);
+  }
   return (PConvAutoNone(result));
 #endif
 }
@@ -292,6 +334,7 @@ static void ObjectMeshStateFree(ObjectMeshState * ms)
   if (ms->shaderCGO){
     CGOFree(ms->shaderCGO);
     ms->shaderCGO = NULL;
+    ms->shaderUnitCellCGO = NULL;
   }
   if(ms->UnitCellCGO) {
     CGOFree(ms->UnitCellCGO);
@@ -361,28 +404,6 @@ void ObjectMeshDump(ObjectMesh * I, char *fname, int state)
   }
 }
 
-#if 0
-static char *ObjectMeshGetCaption(ObjectMesh * I)
-{
-  int state = ObjectGetCurrentState((CObject *) I, false);
-  if(state >= 0) {
-    if(state < I->NState) {
-      char *p;
-      ObjectMeshState *ms = I->State + state;
-      sprintf(ms->caption, "@ %1.4f\n", ms->Level);
-      p = ms->caption + strlen(ms->caption);
-      while(p > ms->caption) {
-        p--;
-        if(*p != '0')
-          break;
-      }
-      return ms->caption;
-    }
-  }
-  return NULL;
-}
-#endif
-
 static void ObjectMeshInvalidate(ObjectMesh * I, int rep, int level, int state)
 {
   int a;
@@ -397,6 +418,7 @@ static void ObjectMeshInvalidate(ObjectMesh * I, int rep, int level, int state)
 	if (ms && ms->shaderCGO){
 	  CGOFree(ms->shaderCGO);
 	  ms->shaderCGO = NULL;
+	  ms->shaderUnitCellCGO = NULL;
 	}
       }
     } else {
@@ -404,6 +426,7 @@ static void ObjectMeshInvalidate(ObjectMesh * I, int rep, int level, int state)
       if (ms && ms->shaderCGO){
 	CGOFree(ms->shaderCGO);
 	ms->shaderCGO = NULL;
+	ms->shaderUnitCellCGO = NULL;
       }
     }
     for(a = 0; a < I->NState; a++) {
@@ -480,7 +503,13 @@ int ObjectMeshSetLevel(ObjectMesh * I, float level, int state, int quiet)
 static void ObjectMeshStateUpdateColors(ObjectMesh * I, ObjectMeshState * ms)
 {
   int one_color_flag = true;
-  int cur_color = SettingGet_color(I->Obj.G, I->Obj.Setting, NULL, cSetting_mesh_color);
+  int cur_color = -1;
+
+  if(ms->MeshMode == 0) {
+    cur_color = SettingGet_color(I->Obj.G, I->Obj.Setting, NULL, cSetting_mesh_color);
+  } else if(ms->MeshMode == 1) {
+    cur_color = SettingGet_color(I->Obj.G, I->Obj.Setting, NULL, cSetting_dot_color);
+  }
 
   if(cur_color == -1)
     cur_color = I->Obj.Color;
@@ -805,6 +834,7 @@ static void ObjectMeshUpdate(ObjectMesh * I)
       if (ms->shaderCGO){
 	CGOFree(ms->shaderCGO);
 	ms->shaderCGO = NULL;
+	ms->shaderUnitCellCGO = NULL;
       }
     }
     SceneInvalidate(I->Obj.G);
@@ -816,50 +846,107 @@ static void ObjectMeshUpdate(ObjectMesh * I)
   }
 }
 
+
 static void ObjectMeshRender(ObjectMesh * I, RenderInfo * info)
+{
+  ObjectMeshRenderImpl(I, info, 0, 0);
+}
+
+static short ObjectMeshStateRenderShader(ObjectMeshState *ms, ObjectMesh *I,
+    RenderInfo *info, short mesh_as_cylinders, float mesh_width)
+{
+  PyMOLGlobals *G = I->Obj.G;
+  CShaderPrg *shaderPrg;
+
+  if (mesh_as_cylinders) {
+    shaderPrg = CShaderPrg_Enable_CylinderShader(G);
+    CShaderPrg_Set1f(shaderPrg, "uni_radius",
+        SceneGetLineWidthForCylinders(G, info, mesh_width));
+  } else {
+    shaderPrg = CShaderPrg_Enable_DefaultShader(G);
+    CShaderPrg_SetLightingEnabled(shaderPrg, 1);
+    CShaderPrg_Set1i(shaderPrg, "two_sided_lighting_enabled",
+		     SceneGetTwoSidedLighting(G));
+  }
+
+  if (!shaderPrg)
+    return false;
+
+  CGORenderGL(ms->shaderCGO, NULL, NULL, NULL, info, NULL);
+  CShaderPrg_Disable(shaderPrg);
+
+  if (ms->shaderUnitCellCGO){
+    shaderPrg = CShaderPrg_Enable_DefaultShader(G);
+    CShaderPrg_SetLightingEnabled(shaderPrg, 0);
+    CGORenderGL(ms->shaderUnitCellCGO, NULL, NULL, NULL, info, NULL);
+    CShaderPrg_Disable(shaderPrg);
+  }
+
+  return true;
+}
+
+static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCGO, int stateArg)
 {
   PyMOLGlobals *G = I->Obj.G;
   float *v = NULL;
   float *vc;
   int *rc;
   float radius;
-  int state = info->state;
-  CRay *ray = info->ray;
-  Picking **pick = info->pick;
-  int pass = info->pass;
+  int state = 0;
+  CRay *ray = 0;
+  Picking **pick = 0;
+  int pass = 0;
   int *n = NULL;
   int c;
   int a = 0;
   float line_width, mesh_width = SettingGet_f(I->Obj.G, I->Obj.Setting, NULL, cSetting_mesh_width);
   ObjectMeshState *ms = NULL;
+  int ok = true;
+
+  if (info){
+    state = info->state;
+    ray = info->ray;
+    pick = info->pick;
+    pass = info->pass;
+  } else {
+    state = stateArg;
+  }
+
   line_width = SceneGetDynamicLineWidth(info, mesh_width);
   ObjectPrepareContext(&I->Obj, ray);
 
-  if(state >= 0)
-    if(state < I->NState)
-      if(I->State[state].Active)
-        if(I->State[state].V && I->State[state].N)
-          ms = I->State + state;
-
-  while(1) {
+  for(a = 0; ok && a < I->NState; a++) {
     if(state < 0) {             /* all_states */
       ms = I->State + a;
-    } else {
-      if(!ms) {
-        if(I->NState &&
-           ((SettingGet(I->Obj.G, cSetting_static_singletons) && (I->NState == 1))))
-          ms = I->State;
-      }
+    } else if(a > 0) {
+      break;
+    } else if(0 <= state && state < I->NState) {
+      ms = I->State + state;
+    } else if(I->NState == 1 && SettingGetGlobal_b(G, cSetting_static_singletons)) {
+      ms = I->State;
     }
-    if(ms) {
-      if(ms->Active && ms->V && ms->N) {
+
+    if(!ms) {
+      ok = false;
+      break;
+    }
+
+    if(!ms->Active || !ms->V || !ms->N)
+      continue;
+
+    {
         v = ms->V;
         n = ms->N;
-        if(ray) {
-
-          if(ms->UnitCellCGO && (I->Obj.RepVis[cRepCell]))
-            CGORenderRay(ms->UnitCellCGO, ray, ColorGet(I->Obj.G, I->Obj.Color),
-                         I->Obj.Setting, NULL);
+        if(ok && ray) {
+          if(ms->UnitCellCGO && (I->Obj.RepVis[cRepCell])){
+            ok &= CGORenderRay(ms->UnitCellCGO, ray, ColorGet(I->Obj.G, I->Obj.Color),
+			       I->Obj.Setting, NULL);
+	    if (!ok){
+	      CGOFree(ms->UnitCellCGO);
+	      ms->UnitCellCGO = NULL;
+	      break;
+	    }
+	  }
           if(ms->MeshMode != 1) {
             radius = SettingGet_f(I->Obj.G, I->Obj.Setting, NULL, cSetting_mesh_radius);
 
@@ -875,18 +962,17 @@ static void ObjectMeshRender(ObjectMesh * I, RenderInfo * info)
             }
           }
 
-          if(n && v && I->Obj.RepVis[cRepMesh]) {
+          if(ok && n && v && I->Obj.RepVis[cRepMesh]) {
             float cc[3];
             float colA[3], colB[3];
             ColorGetEncoded(G, ms->OneColor, cc);
             vc = ms->VC;
             rc = ms->RC;
-            /* vc = ColorGet(I->Obj.G,I->Obj.Color); */
             if(ms->MeshMode == 1) {
               ray->fColor3fv(ray, cc);
-              while(*n) {
+              while(ok && *n) {
                 c = *(n++);
-                while(c--) {
+                while(ok && c--) {
                   if(vc) {
                     register float *cA = vc;
                     if(rc) {
@@ -895,16 +981,16 @@ static void ObjectMeshRender(ObjectMesh * I, RenderInfo * info)
                       rc++;
                     }
                     ray->fColor3fv(ray, cA);
-                    ray->fSphere3fv(ray, v, radius);
+                    ok &= ray->fSphere3fv(ray, v, radius);
                     vc += 3;
                   } else {
-                    ray->fSphere3fv(ray, v, radius);
+                    ok &= ray->fSphere3fv(ray, v, radius);
                   }
                   v += 3;
                 }
               }
             } else {
-              while(*n) {
+              while(ok && *n) {
                 c = *(n++);
                 if(c--) {
                   v += 3;
@@ -913,7 +999,7 @@ static void ObjectMeshRender(ObjectMesh * I, RenderInfo * info)
                     if(rc)
                       rc++;
                   }
-                  while(c--) {
+                  while(ok && c--) {
                     if(vc) {
                       register float *cA = vc - 3, *cB = vc;
                       if(rc) {
@@ -923,10 +1009,10 @@ static void ObjectMeshRender(ObjectMesh * I, RenderInfo * info)
                           ColorGetEncoded(G, rc[0], (cB = colB));
                         rc++;
                       }
-                      ray->fSausage3fv(ray, v - 3, v, radius, cA, cB);
+                      ok &= ray->fSausage3fv(ray, v - 3, v, radius, cA, cB);
                       vc += 3;
                     } else {
-                      ray->fSausage3fv(ray, v - 3, v, radius, cc, cc);
+                      ok &= ray->fSausage3fv(ray, v - 3, v, radius, cc, cc);
                     }
                     v += 3;
                   }
@@ -935,61 +1021,47 @@ static void ObjectMeshRender(ObjectMesh * I, RenderInfo * info)
             }
           }
         } else if(G->HaveGUI && G->ValidContext) {
-          if(pick) {
-          } else {
-            if(!pass) {
-	      short use_shader, generate_shader_cgo = 0, use_display_lists;
+          if(!pick && !pass) {
+	      short use_shader, use_display_lists;
 	      short mesh_as_cylinders ;
-	      use_shader = (int) SettingGet(G, cSetting_mesh_use_shader) & 
-		(int) SettingGet(G, cSetting_use_shaders);
-	      use_display_lists = (int) SettingGet(G, cSetting_use_display_lists);
-	      mesh_as_cylinders = (int) SettingGet(G, cSetting_render_as_cylinders) && SettingGet(G, cSetting_mesh_as_cylinders) && ms->MeshMode != 1;
+	      CGO *shaderCGO = NULL;
+	      use_shader = ( SettingGetGlobal_b(G, cSetting_mesh_use_shader) & SettingGetGlobal_b(G, cSetting_use_shaders)) | returnCGO;
+	      use_display_lists = SettingGetGlobal_i(G, cSetting_use_display_lists);
+	      mesh_as_cylinders = SettingGetGlobal_b(G, cSetting_render_as_cylinders) && SettingGetGlobal_b(G, cSetting_mesh_as_cylinders) && ms->MeshMode != 1;
 
-	      if (ms->shaderCGO && !use_shader){
+	      if (ms->shaderCGO && (!use_shader || (mesh_as_cylinders ^ ms->shaderCGO->has_draw_cylinder_buffers))){
 		CGOFree(ms->shaderCGO);
 		ms->shaderCGO = NULL;
+		ms->shaderUnitCellCGO = NULL;
 	      }
-	      if (ms->shaderCGO && (mesh_as_cylinders ^ ms->shaderCGO->has_draw_cylinder_buffers)){
-		CGOFree(ms->shaderCGO);
-		ms->shaderCGO = NULL;
+
+	      if (ms->shaderCGO && !returnCGO) {
+		ok &= ObjectMeshStateRenderShader(ms, I, info, mesh_as_cylinders, mesh_width);
+		continue;
 	      }
+
 	      if (use_shader){
-		if (!ms->shaderCGO){
-		  ms->shaderCGO = CGONew(G);
-		  ms->shaderCGO->use_shader = true;
-		  generate_shader_cgo = 1;
-		} else {
-		  CShaderPrg *shaderPrg;
-		  if (mesh_as_cylinders){
-		    shaderPrg = CShaderPrg_Enable_CylinderShader(G);
-		    CShaderPrg_Set1f(shaderPrg, "uni_radius", SceneGetLineWidthForCylinders(G, info, mesh_width));
-		  } else {
-		    shaderPrg = CShaderPrg_Enable_DefaultShader(G);
-		    CShaderPrg_Set1i(shaderPrg, "lighting_enabled", 0);
-		  }
-		  CGORenderGL(ms->shaderCGO, NULL, NULL, NULL, info, NULL);
-		  
-		  CShaderPrg_Disable(shaderPrg);
-		  /* instead of returning, need to check if all states are being rendered */
-		  if(state >= 0)
-		    break;
-		  a = a + 1;
-		  if(a >= I->NState)
-		    break;
-		  continue;
+		shaderCGO = CGONew(G);
+		if(!shaderCGO) {
+		  ok = false;
+		  break;
 		}
+		shaderCGO->use_shader = true;
 	      }
 
-	      /* TODO: UnitCellCGO */
-	      if (generate_shader_cgo){
-		CGOColorv(ms->shaderCGO, ColorGet(I->Obj.G, I->Obj.Color));
-		if(ms->UnitCellCGO && (I->Obj.RepVis[cRepCell])){
-		  CGOAppend(ms->shaderCGO, ms->UnitCellCGO);
+	      if(ms->UnitCellCGO && I->Obj.RepVis[cRepCell]) {
+		float *color = ColorGet(I->Obj.G, I->Obj.Color);
+		if (!use_shader) {
+		  CGORenderGL(ms->UnitCellCGO, color, I->Obj.Setting, NULL, info, NULL);
+		} else if(!ms->shaderUnitCellCGO) {
+		  CGO *newUnitCellCGO = CGONewSized(G, 0);
+		  CGOColorv(newUnitCellCGO, color);
+		  CGOAppend(newUnitCellCGO, ms->UnitCellCGO);
+		  ms->shaderUnitCellCGO = CGOOptimizeToVBONotIndexed(newUnitCellCGO, 0);
+		  CGOFree(newUnitCellCGO);
+		  ms->shaderUnitCellCGO->use_shader = true;
+		  ms->shaderUnitCellCGO->enable_shaders = false;
 		}
-	      } else {
-		if(ms->UnitCellCGO && (I->Obj.RepVis[cRepCell]))
-		  CGORenderGL(ms->UnitCellCGO, ColorGet(I->Obj.G, I->Obj.Color),
-			      I->Obj.Setting, NULL, info, NULL);
 	      }
 
 #ifdef _PYMOL_GL_CALLLISTS
@@ -1012,104 +1084,104 @@ static void ObjectMeshRender(ObjectMesh * I, RenderInfo * info)
 	      }
 #endif
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-	      if(!info->line_lighting){
-		if (generate_shader_cgo){
-		  if (!mesh_as_cylinders){
-		    CGODisable(ms->shaderCGO, GL_LIGHTING);
-		  }
-		} else {
+	      if(info && !info->line_lighting){
+		if(!use_shader){
 		  glDisable(GL_LIGHTING);
+		} else if(!mesh_as_cylinders) {
+		  ok &= CGODisable(shaderCGO, GL_LIGHTING);
 		}
 	      }
-#endif
-	      if (generate_shader_cgo){
-		CGOResetNormal(ms->shaderCGO, false);
-		ObjectUseColorCGO(ms->shaderCGO, &I->Obj);
+	      if(!ok) break;
+
+	      if (use_shader){
+		ok &= CGOResetNormal(shaderCGO, false);
 	      } else {
 		SceneResetNormal(I->Obj.G, false);
-		ObjectUseColor(&I->Obj);
 	      }
-
-	      if (generate_shader_cgo){
-		if(n && v && I->Obj.RepVis[cRepMesh]) {
+	      if(n && v && I->Obj.RepVis[cRepMesh]) {
+		if(use_shader) {
 		  vc = ms->VC;
-		  if(ms->MeshMode == 1){
-		    CGODotwidth(ms->shaderCGO, SettingGet_f
-				(I->Obj.G, I->Obj.Setting, NULL, cSetting_dot_width));
-		  } else {
-		    CGOLinewidthSpecial(ms->shaderCGO, LINEWIDTH_DYNAMIC_MESH); 
-		  }
+
+		  if(!vc)
+		    ok &= CGOColorv(shaderCGO, ColorGet(I->Obj.G, ms->OneColor));
+
+		  if (!mesh_as_cylinders){
+		    if(ms->MeshMode == 1){
+		      ok &= CGODotwidth(shaderCGO, SettingGet_f
+				  (I->Obj.G, I->Obj.Setting, NULL, cSetting_dot_width));
+		    } else {
+		      ok &= CGOLinewidthSpecial(shaderCGO, LINEWIDTH_DYNAMIC_MESH); 
+		    }
+		  } 
+
+		  if(!ok) break;
+
 		  if (mesh_as_cylinders){
-		    while(*n) {
-		      float *origin = NULL, *vertex = NULL, *color = NULL, *color2 = NULL, axis[3];
-		      c = *(n++);
-		      while(c--) {
-			origin = vertex;
-			color = color2;
-			if(vc) {
-			  color2 = vc;
+		    if(returnCGO) {
+		      ok &= CGOLinewidthSpecial(shaderCGO, CYLINDERWIDTH_DYNAMIC_MESH);
+		    }
+                    for(; ok && (c = *(n++)); v += 3, vc && (vc += 3)) {
+                      for(; ok && (--c); v += 3) {
+                        float axis[] = {
+                          v[3] - v[0],
+                          v[4] - v[1],
+                          v[5] - v[2]
+                        };
+                        if(vc) {
+                          ok &= CGOColorv(shaderCGO, vc);
 			  vc += 3;
 			}
-			vertex = v;
-			v += 3;
-			if (origin && vertex){
-			  axis[0] = vertex[0] - origin[0];
-			  axis[1] = vertex[1] - origin[1];
-			  axis[2] = vertex[2] - origin[2];
-			  if (vc){
-			    CGOColorv(ms->shaderCGO, color);
-			    if ((*(color) != *(color2) || 
-				     *(color+1) != *(color2+1) || 
-				     *(color+2) != *(color2+2))){
-			      CGOShaderCylinder2ndColor(ms->shaderCGO, origin, axis, 1.f, 15, color2);
-			    } else {
-			      CGOShaderCylinder(ms->shaderCGO, origin, axis, 1.f, 15);
-			    }
-			  } else {
-			    CGOShaderCylinder(ms->shaderCGO, origin, axis, 1.f, 15);
-			  }
-			}
-		      }
+                        if(vc && memcmp(vc - 3, vc, sizeof(float) * 3)) {
+                          ok &= CGOShaderCylinder2ndColor(shaderCGO, v, axis, 1.f, 15, vc);
+                        } else {
+                          ok &= CGOShaderCylinder(shaderCGO, v, axis, 1.f, 15);
+                        }
+                      }
 		    }
 		  } else {
-		    while(*n) {
+		    while(ok && *n) {
 		      c = *(n++);
 		      if(ms->MeshMode == 1)
-			CGOBegin(ms->shaderCGO, GL_POINTS);
-		      else
-			CGOBegin(ms->shaderCGO, GL_LINE_STRIP);
-		      while(c--) {
+			ok &= CGOBegin(shaderCGO, GL_POINTS);
+		      else {
+			if (c < 2){
+			  while(c--) {
+			    if(vc) {
+			      vc += 3;
+			    }
+			    v += 3;
+			  }
+			  continue;
+			}
+			ok &= CGOBegin(shaderCGO, GL_LINE_STRIP);
+		      }
+		      while(ok && c--) {
 			if(vc) {
-			  CGOColorv(ms->shaderCGO, vc);
+			  ok &= CGOColorv(shaderCGO, vc);
 			  vc += 3;
 			}
-			CGOVertexv(ms->shaderCGO, v);
+			if (ok)
+			  ok &= CGOVertexv(shaderCGO, v);
 			v += 3;
 		      }
-		      CGOEnd(ms->shaderCGO);
+		      if (ok)
+			ok &= CGOEnd(shaderCGO);
 		    }
 		  }
-		}
-	      } else {
-		if(n && v && I->Obj.RepVis[cRepMesh]) {
+		} else {
 		  vc = ms->VC;
+
+		  if(!vc)
+		    glColor3fv(ColorGet(I->Obj.G, ms->OneColor));
+
 		  if(ms->MeshMode == 1){
-#ifdef PURE_OPENGL_ES_2
-#else
 		    glPointSize(SettingGet_f
 				(I->Obj.G, I->Obj.Setting, NULL, cSetting_dot_width));
-#endif
 		  } else {
 		    glLineWidth(line_width);
 		  }
 		  while(*n) {
 		    c = *(n++);
-#ifdef PURE_OPENGL_ES_2
-		    /* TODO */
-#else
 #ifdef _PYMOL_GL_DRAWARRAYS
 		    {
 		      int nverts = c;
@@ -1158,41 +1230,45 @@ static void ObjectMeshRender(ObjectMesh * I, RenderInfo * info)
 		      v += 3;
 		    }
 		    glEnd();
-#endif
 #endif
 		  }
 		}
 	      }
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-	      if(!info->line_lighting){
-		if (generate_shader_cgo){
-		  if (!mesh_as_cylinders){
-		    CGOEnable(ms->shaderCGO, GL_LIGHTING);
-		  }
-		} else {
+	      if(info && !info->line_lighting){
+		if(!use_shader){
 		  glEnable(GL_LIGHTING);
+		} else if (ok && !mesh_as_cylinders){
+		  ok &= CGOEnable(shaderCGO, GL_LIGHTING);
 		}
 	      }
-#endif
 
 	      if (use_shader) {
-		if (generate_shader_cgo){
+		if (ok){
 		  CGO *convertcgo = NULL;
-		  CGOStop(ms->shaderCGO);
+		  if (ok)
+		    ok &= CGOStop(shaderCGO);
 #ifdef _PYMOL_CGO_DRAWARRAYS
-		  convertcgo = CGOCombineBeginEnd(ms->shaderCGO, 0);    
-		  CGOFree(ms->shaderCGO);    
-		  ms->shaderCGO = convertcgo;
+		  if (ok)
+		    convertcgo = CGOCombineBeginEnd(shaderCGO, 0);
+		  CHECKOK(ok, convertcgo);
+		  CGOFree(shaderCGO);    
+		  shaderCGO = convertcgo;
 #else
 		  (void)convertcgo;
 #endif
 #ifdef _PYMOL_CGO_DRAWBUFFERS
-		  if (mesh_as_cylinders){
-		    convertcgo = CGOOptimizeGLSLCylindersToVBOIndexed(ms->shaderCGO, 0);
+		  if (returnCGO){
+		    return (shaderCGO);
 		  } else {
-		    convertcgo = CGOOptimizeToVBOIndexed(ms->shaderCGO, 0);
+		    ms->shaderCGO = shaderCGO;
+		  }
+		  if (ok){
+		    if (mesh_as_cylinders){
+		      convertcgo = CGOOptimizeGLSLCylindersToVBOIndexed(ms->shaderCGO, 0);
+		    } else {
+		      convertcgo = CGOOptimizeToVBONotIndexed(ms->shaderCGO, 0);
+		    }
+		    CHECKOK(ok, convertcgo);
 		  }
 		  if (convertcgo){
 		    CGOFree(ms->shaderCGO);
@@ -1203,23 +1279,9 @@ static void ObjectMeshRender(ObjectMesh * I, RenderInfo * info)
 #endif
 		}
 		
-		{
-		  CShaderPrg *shaderPrg;
-		  if (mesh_as_cylinders){
-		    shaderPrg = CShaderPrg_Enable_CylinderShader(G);
-		    CShaderPrg_Set1f(shaderPrg, "uni_radius", SceneGetLineWidthForCylinders(G, info, mesh_width));
-		  } else {
-		    shaderPrg = CShaderPrg_Enable_DefaultShader(G);
-		    CShaderPrg_Set1i(shaderPrg, "lighting_enabled", 1);
-		    CShaderPrg_Set1i(shaderPrg, "two_sided_lighting_enabled", SceneGetTwoSidedLighting(G));
-		  }
-		  {
-		    float *color;
-		    color = ColorGet(G, I->Obj.Color);
-		    CGORenderGL(ms->shaderCGO, color, NULL, NULL, info, NULL);
-		  }
-		  CShaderPrg_Disable(shaderPrg);
-		}
+		if(!ok) break;
+
+                ok &= ObjectMeshStateRenderShader(ms, I, info, mesh_as_cylinders, mesh_width);
 	      }
 	      
 #ifdef _PYMOL_GL_CALLLISTS
@@ -1229,17 +1291,18 @@ static void ObjectMeshRender(ObjectMesh * I, RenderInfo * info)
 	      }
 #endif
       
-	    }
 	  }
 	}
-      }
     }
-    if(state >= 0)
-      break;
-    a = a + 1;
-    if(a >= I->NState)
-      break;
   }
+  if (!ok){
+    I->Obj.fInvalidate(&I->Obj, cRepMesh, cRepInvPurge, -1);
+    I->Obj.fInvalidate(&I->Obj, cRepCGO, cRepInvPurge, -1);
+    ObjectSetRepVis(&I->Obj, cRepMesh, 0);
+    ObjectSetRepVis(&I->Obj, cRepCGO, 0);
+  }
+
+  return NULL;
 }
 
 
@@ -1254,20 +1317,30 @@ static int ObjectMeshGetNStates(ObjectMesh * I)
 /*========================================================================*/
 ObjectMesh *ObjectMeshNew(PyMOLGlobals * G)
 {
+  int ok = true;
   OOAlloc(G, ObjectMesh);
+  CHECKOK(ok, I);
+  if (ok)
+    ObjectInit(G, (CObject *) I);
 
-  ObjectInit(G, (CObject *) I);
-
-  I->NState = 0;
-  I->State = VLAMalloc(10, sizeof(ObjectMeshState), 5, true);   /* autozero important */
-
-  I->Obj.type = cObjectMesh;
-
-  I->Obj.fFree = (void (*)(CObject *)) ObjectMeshFree;
-  I->Obj.fUpdate = (void (*)(CObject *)) ObjectMeshUpdate;
-  I->Obj.fRender = (void (*)(CObject *, RenderInfo *)) ObjectMeshRender;
-  I->Obj.fInvalidate = (void (*)(CObject *, int, int, int)) ObjectMeshInvalidate;
-  I->Obj.fGetNFrame = (int (*)(CObject *)) ObjectMeshGetNStates;
+  if (ok){
+    I->NState = 0;
+    I->State = VLACalloc(ObjectMeshState, 10);   /* autozero important */
+    CHECKOK(ok, I->State);
+  }
+  if (ok){
+    I->Obj.type = cObjectMesh;
+    
+    I->Obj.fFree = (void (*)(CObject *)) ObjectMeshFree;
+    I->Obj.fUpdate = (void (*)(CObject *)) ObjectMeshUpdate;
+    I->Obj.fRender = (void (*)(CObject *, RenderInfo *)) ObjectMeshRender;
+    I->Obj.fInvalidate = (void (*)(CObject *, int, int, int)) ObjectMeshInvalidate;
+    I->Obj.fGetNFrame = (int (*)(CObject *)) ObjectMeshGetNStates;
+  }
+  if (!ok){
+    ObjectMeshFree(I);
+    I = NULL;
+  }
   /*  I->Obj.fGetCaption = (char *(*)(CObject *))ObjectMeshGetCaption; */
   return (I);
 }
@@ -1307,6 +1380,7 @@ void ObjectMeshStateInit(PyMOLGlobals * G, ObjectMeshState * ms)
   ms->caption[0] = 0;
   ms->Field = NULL;
   ms->shaderCGO = NULL;
+  ms->shaderUnitCellCGO = NULL;
 }
 
 
@@ -1320,36 +1394,45 @@ ObjectMesh *ObjectMeshFromXtalSym(PyMOLGlobals * G, ObjectMesh * obj, ObjectMap
                                   float alt_level, int quiet)
 {
   int ok = true;
-  ObjectMesh *I;
-  ObjectMeshState *ms;
-  ObjectMapState *oms;
+  ObjectMesh *I = NULL;
+  ObjectMeshState *ms = NULL;
+  ObjectMapState *oms = NULL;
+  int created = !obj;
 
-  if(!obj) {
+  if(created) {
     I = ObjectMeshNew(G);
   } else {
     I = obj;
   }
+  CHECKOK(ok, I);
 
-  if(state < 0)
-    state = I->NState;
-  if(I->NState <= state) {
-    VLACheck(I->State, ObjectMeshState, state);
-    I->NState = state + 1;
+  if (ok){
+    if(state < 0)
+      state = I->NState;
+    if(I->NState <= state) {
+      VLACheck(I->State, ObjectMeshState, state);
+      CHECKOK(ok, I->State);
+      if (ok)
+	I->NState = state + 1;
+    }
   }
 
-  ms = I->State + state;
-  ObjectMeshStateInit(G, ms);
+  if (ok){
+    ms = I->State + state;
+    ObjectMeshStateInit(G, ms);
+  }
 
-  strcpy(ms->MapName, map->Obj.Name);
-  ms->MapState = map_state;
-  oms = ObjectMapGetState(map, map_state);
-
-  ms->Level = level;
-  ms->AltLevel = alt_level;
-  ms->MeshMode = meshMode;
-  ms->quiet = quiet;
-  if(oms) {
+  if (ok){
+    strcpy(ms->MapName, map->Obj.Name);
+    ms->MapState = map_state;
+    oms = ObjectMapGetState(map, map_state);
 
+    ms->Level = level;
+    ms->AltLevel = alt_level;
+    ms->MeshMode = meshMode;
+    ms->quiet = quiet;
+  }
+  if(ok && oms) {
     if((meshMode == 3) && (ms->AltLevel < ms->Level)) {
       /* gradient object -- need to auto-set range */
       if(!ObjectMapStateGetDataRange(G, oms, &ms->Level, &ms->AltLevel)) {
@@ -1362,12 +1445,12 @@ ObjectMesh *ObjectMeshFromXtalSym(PyMOLGlobals * G, ObjectMesh * obj, ObjectMap
     copy3f(mx, ms->ExtentMax);
 
     if(oms->State.Matrix) {
-      ObjectStateSetMatrix(&ms->State, oms->State.Matrix);
+      ok &= ObjectStateSetMatrix(&ms->State, oms->State.Matrix);
     } else if(ms->State.Matrix) {
       ObjectStateResetMatrix(&ms->State);
     }
 
-    {
+    if (ok) {
       float *min_ext, *max_ext;
       float tmp_min[3], tmp_max[3];
       if(MatrixInvTransformExtentsR44d3f(ms->State.Matrix,
@@ -1380,7 +1463,7 @@ ObjectMesh *ObjectMeshFromXtalSym(PyMOLGlobals * G, ObjectMesh * obj, ObjectMap
         max_ext = ms->ExtentMax;
       }
 
-      {
+      if(sym) {
         int eff_range[6];
 
         if(IsosurfGetRange
@@ -1426,9 +1509,10 @@ ObjectMesh *ObjectMeshFromXtalSym(PyMOLGlobals * G, ObjectMesh * obj, ObjectMap
             ms->Range[a] = eff_range[a];
           }
         }
+      } else {
+        IsosurfGetRange(G, oms->Field, oms->Symmetry->Crystal, min_ext, max_ext, ms->Range, true);
       }
     }
-    ms->ExtentFlag = true;
   }
   if(ok) {
     if(carve != 0.0) {
@@ -1442,8 +1526,8 @@ ObjectMesh *ObjectMeshFromXtalSym(PyMOLGlobals * G, ObjectMesh * obj, ObjectMap
     I->Obj.ExtentFlag = true;
     /*  printf("Brick %d %d %d %d %d %d\n",I->Range[0],I->Range[1],I->Range[2],I->Range[3],I->Range[4],I->Range[5]); */
   }
-  if(!ok) {
-    ObjectMeshStateFree(ms);
+  if(!ok && created) {
+    ObjectMeshFree(I);
     I = NULL;
   }
   SceneChanged(G);
@@ -1459,83 +1543,8 @@ ObjectMesh *ObjectMeshFromBox(PyMOLGlobals * G, ObjectMesh * obj, ObjectMap * ma
                               float level, int meshMode,
                               float carve, float *vert_vla, float alt_level, int quiet)
 {
-  ObjectMesh *I;
-  ObjectMeshState *ms;
-  ObjectMapState *oms;
-
-  if(!obj) {
-    I = ObjectMeshNew(G);
-  } else {
-    I = obj;
-  }
-
-  if(state < 0)
-    state = I->NState;
-  if(I->NState <= state) {
-    VLACheck(I->State, ObjectMeshState, state);
-    I->NState = state + 1;
-  }
-
-  ms = I->State + state;
-  ObjectMeshStateInit(G, ms);
-
-  strcpy(ms->MapName, map->Obj.Name);
-  ms->MapState = map_state;
-  oms = ObjectMapGetState(map, map_state);
-
-  ms->Level = level;
-  ms->AltLevel = alt_level;
-  ms->MeshMode = meshMode;
-  ms->quiet = quiet;
-  if(oms) {
-
-    if((meshMode == 3) && (ms->AltLevel < ms->Level)) {
-      /* gradient object -- need to auto-set range */
-      if(!ObjectMapStateGetDataRange(G, oms, &ms->Level, &ms->AltLevel)) {
-        ms->Level = -1.0F;
-        ms->AltLevel = 1.0F;
-      }
-    }
-
-    copy3f(mn, ms->ExtentMin);  /* this is not exactly correct...should actually take vertex points from range */
-    copy3f(mx, ms->ExtentMax);
-
-    if(oms->State.Matrix) {
-      ObjectStateSetMatrix(&ms->State, oms->State.Matrix);
-    } else if(ms->State.Matrix) {
-      ObjectStateResetMatrix(&ms->State);
-    }
-
-    {
-      float *min_ext, *max_ext;
-      float tmp_min[3], tmp_max[3];
-      if(MatrixInvTransformExtentsR44d3f(ms->State.Matrix,
-                                         ms->ExtentMin, ms->ExtentMax,
-                                         tmp_min, tmp_max)) {
-        min_ext = tmp_min;
-        max_ext = tmp_max;
-      } else {
-        min_ext = ms->ExtentMin;
-        max_ext = ms->ExtentMax;
-      }
-
-      IsosurfGetRange(G, oms->Field, oms->Symmetry->Crystal, min_ext, max_ext, ms->Range, true);
-    }
-    ms->ExtentFlag = true;
-  }
-  if(carve != 0.0) {
-    ms->CarveFlag = true;
-    ms->CarveBuffer = carve;
-    ms->AtomVertex = vert_vla;
-  }
-  if(I) {
-    ObjectMeshRecomputeExtent(I);
-  }
-  I->Obj.ExtentFlag = true;
-  /*  printf("Brick %d %d %d %d %d %d\n",I->Range[0],I->Range[1],I->Range[2],I->Range[3],I->Range[4],I->Range[5]); */
-  SceneChanged(G);
-  SceneCountFrames(G);
-  return (I);
+  return ObjectMeshFromXtalSym(G, obj, map, NULL, map_state, state, mn, mx,
+      level, meshMode, carve, vert_vla, alt_level, quiet);
 }
 
 
