@@ -131,25 +131,6 @@ static float ShakerDoTors(int type, float *v0, float *v1, float *v2, float *v3,
   normalize3f(perp0);
   normalize3f(perp1);
 
-#if 0
-  {
-    float vert[3];
-    /* debuggin */
-
-    CGOColor(DebugCGO, 1.0, 0.0, 0.0);
-    CGOBegin(DebugCGO, GL_LINES);
-    CGOVertexv(DebugCGO, v1);
-    add3f(perp0, v1, vert);
-    CGOVertexv(DebugCGO, vert);
-
-    CGOColor(DebugCGO, 0.0, 1.0, 1.0);
-    CGOVertexv(DebugCGO, v2);
-    add3f(perp1, v2, vert);
-    CGOVertexv(DebugCGO, vert);
-    CGOEnd(DebugCGO);
-  }
-#endif
-
   dp = dot_product3f(perp0, perp1);
 
   switch (type) {
@@ -160,8 +141,6 @@ static float ShakerDoTors(int type, float *v0, float *v1, float *v2, float *v3,
         result = result / 5.0F;
     } else if(dp < 0.5) {
       result = -0.5F - dp;
-      if(result < tole)         /* discontinuous low bottom well */
-        result = result;
     } else {
       result = 1.0F - dp;
     }
@@ -209,30 +188,6 @@ static float ShakerDoTors(int type, float *v0, float *v1, float *v2, float *v3,
   subtract3f(p1, push0, p1);
   subtract3f(p2, push3, p2);
 
-#if 0
-  {
-    float vert[3];
-    /* debuggin */
-
-    CGOColor(DebugCGO, 1.0, 0.5, 0.0);
-    CGOBegin(DebugCGO, GL_LINES);
-    CGOLinewidth(DebugCGO, 3.0);
-
-    /* draw from */
-
-    CGOVertexv(DebugCGO, v0);
-    add3f(v0, push0, vert);
-    CGOVertexv(DebugCGO, vert);
-
-    /* draw to */
-
-    CGOVertexv(DebugCGO, v3);
-    add3f(v3, push3, vert);
-    CGOVertexv(DebugCGO, vert);
-    CGOEnd(DebugCGO);
-  }
-#endif
-
   return result;
 
 }
@@ -1302,19 +1257,7 @@ void SculptMeasureObject(CSculpt * I, ObjectMolecule * obj, int state, int match
 
                             d = (float) diff3f(v1, v3);
 
-#if 0
-                            if(use_cache) {
-                              if(!SculptCacheQuery(G, cSculptBond,
-                                                   obj_atomInfo[b1].unique_id,
-                                                   obj_atomInfo[b2].unique_id, 0, 0, &d))
-                                SculptCacheStore(G, cSculptBond,
-                                                 obj_atomInfo[b1].unique_id,
-                                                 obj_atomInfo[b2].unique_id, 0, 0, d);
-                            }
-#endif
-
                             ShakerAddDistCon(I->Shaker, b1, b3, d, cShakerDistBond, 1.0F);
-
                           }
                         }
                       }
@@ -2498,6 +2441,11 @@ float SculptIterateObject(CSculpt * I, ObjectMolecule * obj,
         if(est) {
           cs->SculptCGO = CGOSimplify(cgo, est);
           CGOFree(cgo);
+
+          if (cs->SculptShaderCGO){
+            CGOFree(cs->SculptShaderCGO);
+            cs->SculptShaderCGO = NULL;
+          }
         }
       }
     }
@@ -2523,221 +2471,3 @@ void SculptFree(CSculpt * I)
   ShakerFree(I->Shaker);
   OOFreeP(I);
 }
-
-#if 0
-
-if(ex1 == 4) {                  /* special handling for 1-4's */
-  ex14_b0 = *(j + 4);           /* b1\b0_b2/b3 atoms */
-  ex14_b2 = *(j + 5);
-}
-
-if(SculptDoBump14(cutoff, len, diff,
-                  disp + b0 * 3, disp + b1 * 3,
-                  wt, &total_strain,
-                  v0,
-                  v1,
-                  cs->Coord + 3 * atm2idx[ex14_b0],
-                  cs->Coord + 3 * atm2idx[ex14_b2],
-                  disp + ex14_b0 * 3, disp + ex14_b2 * 3)) {
-  cnt[b0]++;
-  cnt[b1]++;
-  total_count++;
-}
-}
-
-#ifdef _PYMOL_INLINE
-__inline__
-#endif
-static int SculptDoBump14(float target, float actual, float *d,
-                          float *d0to1, float *d1to0, float wt, float *strain,
-                          float *v0, float *v1, float *v2, float *v3,
-                          float *d2to3, float *d3to2)
-{
-  /* v0\v2_v3/v1 */
-
-  float push0[3], push1[3], balance[3];
-  float axis[3], seg0[3], seg1[3], perp0[3], perp1[3];
-  float dir[3];
-  float dev, dev_2, sc, abs_dev;
-  float sign, dp;
-
-  dev = target - actual;
-  if((abs_dev = fabs(dev)) > R_SMALL8) {
-
-    /* v0        v1
-       \      /
-       v2__v3 */
-
-    subtract3f(v3, v2, axis);
-    subtract3f(v0, v2, seg0);
-    normalize3f(axis);
-    subtract3f(v1, v3, seg1);
-    cross_product3f(seg0, axis, perp0);
-    cross_product3f(axis, seg1, perp1);
-
-    normalize3f(perp0);
-    normalize3f(perp1);
-
-#if 0
-    {
-      float vert[3];
-      /* debuggin */
-
-      CGOColor(DebugCGO, 1.0, 0.0, 0.0);
-      CGOBegin(DebugCGO, GL_LINES);
-      CGOVertexv(DebugCGO, v2);
-      add3f(perp0, v2, vert);
-      CGOVertexv(DebugCGO, vert);
-
-      CGOColor(DebugCGO, 0.0, 1.0, 1.0);
-      CGOVertexv(DebugCGO, v3);
-      add3f(perp1, v3, vert);
-      CGOVertexv(DebugCGO, vert);
-      CGOEnd(DebugCGO);
-    }
-#endif
-
-    if((dp = dot_product3f(perp0, perp1)) > -0.5F)
-      return 0;
-
-    dev_2 = wt * dev / 2.0F;
-    (*strain) += abs_dev;
-    if(actual > R_SMALL8) {     /* nonoverlapping */
-
-      cross_product3f(perp0, perp1, dir);
-      sign = dot_product3f(axis, dir);
-
-      if(sign < 0.0F)
-        sc = wt * (fabs(dp) - 0.5);
-      else
-        sc = wt * (0.5 - fabs(dp));
-
-      scale3f(perp0, sc, push0);
-      scale3f(perp1, sc, push1);
-      add3f(push0, push1, balance);
-      scale3f(balance, -0.5, balance);
-
-#if 0
-      {
-        float vert[3];
-        /* debuggin */
-
-        CGOColor(DebugCGO, 1.0, 0.5, 0.0);
-        CGOBegin(DebugCGO, GL_LINES);
-        CGOLinewidth(DebugCGO, 3.0);
-
-        /* draw from */
-
-        CGOVertexv(DebugCGO, v0);
-        add3f(v0, push0, vert);
-        CGOVertexv(DebugCGO, vert);
-
-        /* draw to */
-
-        CGOVertexv(DebugCGO, v1);
-        add3f(v1, push1, vert);
-        CGOVertexv(DebugCGO, vert);
-
-        CGOVertexv(DebugCGO, v2);
-        add3f(v2, balance, vert);
-        CGOVertexv(DebugCGO, vert);
-
-        CGOVertexv(DebugCGO, v3);
-        add3f(v3, balance, vert);
-        CGOVertexv(DebugCGO, vert);
-
-        CGOEnd(DebugCGO);
-
-      }
-#endif
-
-      add3f(d0to1, push0, d0to1);
-      add3f(d1to0, push1, d1to0);
-      subtract3f(d2to3, push0, d2to3);
-      subtract3f(d3to2, push1, d3to2);
-
-    } else {                    /* overlapping, so just push along X */
-      d0to1[0] -= dev_2;
-      d1to0[0] += dev_2;
-    }
-    return 1;
-  }
-  return 0;
-}
-
-#endif
-
-#if 0
-{
-  int a4, b4, n3;
-  float *v4;
-
-  /* add additional long-range distance restraints into object (INEFFICIENT) */
-
-  /* b1-b0-b2-b3-b4 */
-
-  for(b0 = 0; b0 < obj->NAtom; b0++) {
-    n0 = neighbor[b0] + 1;
-    while(neighbor[n0] >= 0) {
-      b1 = neighbor[n0];
-      n1 = neighbor[b0] + 1;
-      while(neighbor[n1] >= 0) {
-        b2 = neighbor[n1];
-        if(b1 != b2) {
-          n2 = neighbor[b2] + 1;
-          while(neighbor[n2] >= 0) {
-            b3 = neighbor[n2];
-            if(b3 != b0) {
-              n3 = neighbor[b3] + 1;
-              while(neighbor[n3] >= 0) {
-                b4 = neighbor <[n3];
-                if(b2 != b4) {
-
-                  if(obj->DiscreteFlag) {
-                    if((cs == obj->DiscreteCSet[b0]) &&
-                       (cs == obj->DiscreteCSet[b1]) &&
-                       (cs == obj->DiscreteCSet[b2]) &&
-                       (cs == obj->DiscreteCSet[b3]) && (cs == obj->DiscreteCSet[b4])
-                      ) {
-                      a0 = obj->DiscreteAtmToIdx[b0];
-                      a1 = obj->DiscreteAtmToIdx[b1];
-                      a2 = obj->DiscreteAtmToIdx[b2];
-                      a3 = obj->DiscreteAtmToIdx[b3];
-                      a4 = obj->DiscreteAtmToIdx[b4];
-                    } else {
-                      a0 = -1;
-                      a1 = -1;
-                      a2 = -1;
-                      a3 = -1;
-                      a4 = -1;
-                    }
-                  } else {
-                    a0 = cs->AtmToIdx[b0];
-                    a1 = cs->AtmToIdx[b1];
-                    a2 = cs->AtmToIdx[b2];
-                    a3 = cs->AtmToIdx[b3];
-                    a4 = cs->AtmToIdx[b4];
-                  }
-                  if((a0 >= 0) && (a1 >= 0) && (a2 >= 0) && (a3 >= 0) && (a4 >= 0)) {
-                    v0 = cs->Coord + 3 * a0;
-                    v1 = cs->Coord + 3 * a1;
-                    v2 = cs->Coord + 3 * a2;
-                    v3 = cs->Coord + 3 * a3;
-                    v4 = cs->Coord + 3 * a4;
-                    d = diff3f(v0, v1) + diff3f(v0, v2) + diff3f(v2, v3) + diff3f(v3, v4);
-                    ShakerAddDistCon(I->Shaker, b0, b1, d, cShakerDistLimit, 1.0F);
-                  }
-                }
-                n3 += 2;
-              }
-            }
-            n2 += 2;
-          }
-        }
-        n1 += 2;
-      }
-      n0 += 2;
-    }
-  }
-}
-#endif
