@@ -33,6 +33,7 @@ Z* -------------------------------------------------------------------
 #include "Color.h"
 #include "Text.h"
 #include "PyMOL.h"
+#include "CGO.h"
 
 #define cPopUpLineHeight 17
 #define cPopUpTitleHeight 19
@@ -68,10 +69,42 @@ typedef struct CPopUp {
 } CPopUp;
 
 int PopUpRelease(Block * block, int button, int x, int y, int mod);
-void PopUpDraw(Block * block);
+void PopUpDraw(Block * block ORTHOCGOARG);
 int PopUpDrag(Block * block, int x, int y, int mod);
 int PopUpConvertY(CPopUp * I, int value, int mode);
 
+/*========================================================================
+ * If Sub[a] is a list, return it, otherwise assume it's callable so
+ * call it, assign the result to Sub[a] and return the result.
+ *
+ * @param Sub : array of lists or callables
+ * @param a   : array index
+ */
+static PyObject * SubGetItem(PyMOLGlobals * G, PyObject ** Sub, const int a) {
+#ifndef _PYMOL_NOPY
+  PyObject *elem = Sub[a];
+  th_assert(1, elem);
+
+  if(!PyList_Check(elem)) {
+    PBlock(G);
+    elem = PyObject_CallObject(elem, NULL);
+    PUnblock(G);
+
+    th_assert(2, elem);
+
+    Py_DECREF(Sub[a]);
+    Py_INCREF(Sub[a] = elem);
+  }
+
+  return elem;
+
+th_except2:
+  if(PyErr_Occurred())
+    PyErr_Print();
+th_except1:
+#endif
+  return NULL;
+}
 
 /*========================================================================*/
 static Block *PopUpRecursiveFind(Block * block, int x, int y)
@@ -198,7 +231,7 @@ Block *PopUpNew(PyMOLGlobals * G, int x, int y, int last_x, int last_y,
     if(command) {
       if(PyString_Check(command)) {
 	strcpy(I->Command[a], PyString_AsString(command));
-      } else if(PyList_Check(command)) {
+      } else {
 	Py_INCREF(command);
 	I->Sub[a] = command;
       }
@@ -226,6 +259,7 @@ Block *PopUpNew(PyMOLGlobals * G, int x, int y, int last_x, int last_y,
 
   PAutoUnblock(G, blocked);
 
+  OrthoInvalidateDoDraw(G);  
   return I->Block;
 #endif
 
@@ -478,6 +512,7 @@ int PopUpDrag(Block * block, int x, int y, int mod)
             I->Child = NULL;
             I->ChildLine = -1;
             OrthoDirty(G);
+	    OrthoInvalidateDoDraw(G);
           } else {
             I->Selected = a;
           }
@@ -487,15 +522,18 @@ int PopUpDrag(Block * block, int x, int y, int mod)
 
       if(I->Code[a] != 1)
         I->Selected = -1;
-      else if(I->Sub[a]) {
+      else {
         /* activate submenu */
-        if(!I->Child) {
+        PyObject * sub_a = SubGetItem(G, I->Sub, a);
+        if(!sub_a) {
+          // do nothing
+        } else if(!I->Child) {
           I->ChildLine = a;
           if(I->ChildDelay > UtilGetSeconds(G)) {
             PyMOL_NeedFakeDrag(G->PyMOL);       /* keep coming back here... */
           } else {
             I->Child = PopUpNew(G, I->LastX - 300, I->LastY, I->LastX, I->LastY,
-                                false, I->Sub[a], I->Block);
+                                false, sub_a, I->Block);
             {
               int target_y =
                 block->rect.top - (PopUpConvertY(I, a, true) + cPopUpCharMargin);
@@ -516,8 +554,7 @@ int PopUpDrag(Block * block, int x, int y, int mod)
           I->ChildDelay = UtilGetSeconds(G) + cChildDelay;      /* keep child here for a while */
         }
         I->Selected = a;
-      } else
-        I->Selected = a;
+      }
     }
   }
   /* delay updates, etc. so that child menus 
@@ -539,19 +576,22 @@ int PopUpDrag(Block * block, int x, int y, int mod)
       I->DirtyDelayFlag = true;
       I->DirtyDelay = UtilGetSeconds(G) + cDirtyDelay;
     }
-    if(!I->DirtyDelayFlag)
+    if(!I->DirtyDelayFlag){
       OrthoDirty(G);
+      OrthoInvalidateDoDraw(G);
+    }
   }
   if(I->DirtyDelayFlag && (I->DirtyDelay < UtilGetSeconds(G))) {
     I->DirtyDelayFlag = false;
     OrthoDirty(G);
+    OrthoInvalidateDoDraw(G);
   }
   return (1);
 }
 
 
 /*========================================================================*/
-void PopUpDraw(Block * block)
+void PopUpDraw(Block * block ORTHOCGOARG)
 {
   CPopUp *I = (CPopUp *) block->reference;
   PyMOLGlobals *G = block->G;
@@ -565,239 +605,164 @@ void PopUpDraw(Block * block)
 
     /* put raised border around pop-up menu */
 
-    /* bottom */
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3f(0.2F, 0.2F, 0.4F);
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	block->rect.left - 2, block->rect.bottom - 2,
-	block->rect.right + 2, block->rect.bottom - 2,
-	block->rect.left - 2, block->rect.bottom + 1,
-	block->rect.right + 2, block->rect.bottom + 1
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
+    if (orthoCGO){
+      CGOColor(orthoCGO, 0.2F, 0.2F, 0.4F);
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, block->rect.left - 2, block->rect.bottom - 2, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 2, block->rect.bottom - 2, 0.f);
+      CGOVertex(orthoCGO, block->rect.left - 2, block->rect.bottom + 1, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 2, block->rect.bottom + 1, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glColor3f(0.2F, 0.2F, 0.4F);
+      glBegin(GL_POLYGON);
+      glVertex2i(block->rect.left - 2, block->rect.bottom - 2);
+      glVertex2i(block->rect.right + 2, block->rect.bottom - 2);
+      glVertex2i(block->rect.right + 2, block->rect.bottom + 1);
+      glVertex2i(block->rect.left - 2, block->rect.bottom + 1);
+      glEnd();
     }
-#else
-    glBegin(GL_POLYGON);
-    glVertex2i(block->rect.left - 2, block->rect.bottom - 2);
-    glVertex2i(block->rect.right + 2, block->rect.bottom - 2);
-    glVertex2i(block->rect.right + 2, block->rect.bottom + 1);
-    glVertex2i(block->rect.left - 2, block->rect.bottom + 1);
-    glEnd();
-#endif
-#endif
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3f(0.4F, 0.4F, 0.6F);
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	block->rect.left - 1, block->rect.bottom - 1,
-	block->rect.right + 1, block->rect.bottom - 1,
-	block->rect.left - 1, block->rect.bottom + 1,
-	block->rect.right + 1, block->rect.bottom + 1
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
+    if (orthoCGO){
+      CGOColor(orthoCGO, 0.4F, 0.4F, 0.6F);
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, block->rect.left - 1, block->rect.bottom - 1, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 1, block->rect.bottom - 1, 0.f);
+      CGOVertex(orthoCGO, block->rect.left - 1, block->rect.bottom + 1, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 1, block->rect.bottom + 1, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glColor3f(0.4F, 0.4F, 0.6F);
+      glBegin(GL_POLYGON);
+      glVertex2i(block->rect.left - 1, block->rect.bottom - 1);
+      glVertex2i(block->rect.right + 1, block->rect.bottom - 1);
+      glVertex2i(block->rect.right + 1, block->rect.bottom + 1);
+      glVertex2i(block->rect.left - 1, block->rect.bottom + 1);
+      glEnd();
     }
-#else
-    glBegin(GL_POLYGON);
-    glVertex2i(block->rect.left - 1, block->rect.bottom - 1);
-    glVertex2i(block->rect.right + 1, block->rect.bottom - 1);
-    glVertex2i(block->rect.right + 1, block->rect.bottom + 1);
-    glVertex2i(block->rect.left - 1, block->rect.bottom + 1);
-    glEnd();
-#endif
-#endif
     /* right */
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3f(0.2F, 0.2F, 0.4F);
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	block->rect.right, block->rect.bottom - 2,
-	block->rect.right + 2, block->rect.bottom - 2,
-	block->rect.right, block->rect.top,
-	block->rect.right + 2, block->rect.top
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
+    if (orthoCGO){
+      CGOColor(orthoCGO, 0.2F, 0.2F, 0.4F);
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, block->rect.right, block->rect.bottom - 2, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 2, block->rect.bottom - 2, 0.f);
+      CGOVertex(orthoCGO, block->rect.right, block->rect.top, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 2, block->rect.top, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glColor3f(0.2F, 0.2F, 0.4F);
+      glBegin(GL_POLYGON);
+      glVertex2i(block->rect.right, block->rect.bottom - 2);
+      glVertex2i(block->rect.right + 2, block->rect.bottom - 2);
+      glVertex2i(block->rect.right + 2, block->rect.top);
+      glVertex2i(block->rect.right, block->rect.top);
+      glEnd();
     }
-#else
-    glBegin(GL_POLYGON);
-    glVertex2i(block->rect.right, block->rect.bottom - 2);
-    glVertex2i(block->rect.right + 2, block->rect.bottom - 2);
-    glVertex2i(block->rect.right + 2, block->rect.top);
-    glVertex2i(block->rect.right, block->rect.top);
-    glEnd();
-#endif
-#endif
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3f(0.4F, 0.4F, 0.6F);
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	block->rect.right, block->rect.bottom - 1,
-	block->rect.right + 1, block->rect.bottom - 1,
-	block->rect.right, block->rect.top,
-	block->rect.right + 1, block->rect.top
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
+    if (orthoCGO){
+      CGOColor(orthoCGO, 0.4F, 0.4F, 0.6F);
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, block->rect.right, block->rect.bottom - 1, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 1, block->rect.bottom - 1, 0.f);
+      CGOVertex(orthoCGO, block->rect.right, block->rect.top, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 1, block->rect.top, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glColor3f(0.4F, 0.4F, 0.6F);
+      glBegin(GL_POLYGON);
+      glVertex2i(block->rect.right, block->rect.bottom - 1);
+      glVertex2i(block->rect.right + 1, block->rect.bottom - 1);
+      glVertex2i(block->rect.right + 1, block->rect.top);
+      glVertex2i(block->rect.right, block->rect.top);
+      glEnd();
     }
-#else
-    glBegin(GL_POLYGON);
-    glVertex2i(block->rect.right, block->rect.bottom - 1);
-    glVertex2i(block->rect.right + 1, block->rect.bottom - 1);
-    glVertex2i(block->rect.right + 1, block->rect.top);
-    glVertex2i(block->rect.right, block->rect.top);
-    glEnd();
-#endif
-#endif
     /* top */
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3f(0.5F, 0.5F, 0.7F);
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	block->rect.left - 2, block->rect.top + 2,
-	block->rect.right + 2, block->rect.top + 2,
-	block->rect.left - 2, block->rect.top,
-	block->rect.right + 2, block->rect.top
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
+    if (orthoCGO){
+      CGOColor(orthoCGO, 0.5F, 0.5F, 0.7F);
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, block->rect.left - 2, block->rect.top + 2, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 2, block->rect.top + 2, 0.f);
+      CGOVertex(orthoCGO, block->rect.left - 2, block->rect.top, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 2, block->rect.top, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glColor3f(0.5F, 0.5F, 0.7F);
+      glBegin(GL_POLYGON);
+      glVertex2i(block->rect.left - 2, block->rect.top + 2);
+      glVertex2i(block->rect.right + 2, block->rect.top + 2);
+      glVertex2i(block->rect.right + 2, block->rect.top);
+      glVertex2i(block->rect.left - 2, block->rect.top);
+      glEnd();
     }
-#else
-    glBegin(GL_POLYGON);
-    glVertex2i(block->rect.left - 2, block->rect.top + 2);
-    glVertex2i(block->rect.right + 2, block->rect.top + 2);
-    glVertex2i(block->rect.right + 2, block->rect.top);
-    glVertex2i(block->rect.left - 2, block->rect.top);
-    glEnd();
-#endif
-#endif
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3f(0.6F, 0.6F, 0.8F);
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	block->rect.left - 1, block->rect.top + 1,
-	block->rect.right + 1, block->rect.top + 1,
-	block->rect.left - 1, block->rect.top,
-	block->rect.right + 1, block->rect.top
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
+    if (orthoCGO){
+      CGOColor(orthoCGO, 0.6F, 0.6F, 0.8F);
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, block->rect.left - 1, block->rect.top + 1, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 1, block->rect.top + 1, 0.f);
+      CGOVertex(orthoCGO, block->rect.left - 1, block->rect.top, 0.f);
+      CGOVertex(orthoCGO, block->rect.right + 1, block->rect.top, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glColor3f(0.6F, 0.6F, 0.8F);
+      glBegin(GL_POLYGON);
+      glVertex2i(block->rect.left - 1, block->rect.top + 1);
+      glVertex2i(block->rect.right + 1, block->rect.top + 1);
+      glVertex2i(block->rect.right + 1, block->rect.top);
+      glVertex2i(block->rect.left - 1, block->rect.top);
+      glEnd();
     }
-#else
-    glBegin(GL_POLYGON);
-    glVertex2i(block->rect.left - 1, block->rect.top + 1);
-    glVertex2i(block->rect.right + 1, block->rect.top + 1);
-    glVertex2i(block->rect.right + 1, block->rect.top);
-    glVertex2i(block->rect.left - 1, block->rect.top);
-    glEnd();
-#endif
-#endif
 
     /* left */
-
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3f(0.5F, 0.5F, 0.7F);
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	block->rect.left - 2, block->rect.bottom - 2,
-	block->rect.left, block->rect.bottom,
-	block->rect.left - 2, block->rect.top,
-	block->rect.left, block->rect.top
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
+    if (orthoCGO){
+      CGOColor(orthoCGO, 0.5F, 0.5F, 0.7F);
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, block->rect.left - 2, block->rect.bottom - 2, 0.f);
+      CGOVertex(orthoCGO, block->rect.left, block->rect.bottom, 0.f);
+      CGOVertex(orthoCGO, block->rect.left - 2, block->rect.top, 0.f);
+      CGOVertex(orthoCGO, block->rect.left, block->rect.top, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glColor3f(0.5F, 0.5F, 0.7F);
+      glBegin(GL_POLYGON);
+      glVertex2i(block->rect.left - 2, block->rect.bottom - 2);
+      glVertex2i(block->rect.left, block->rect.bottom);
+      glVertex2i(block->rect.left, block->rect.top);
+      glVertex2i(block->rect.left - 2, block->rect.top);
+      glEnd();
     }
-#else
-    glBegin(GL_POLYGON);
-    glVertex2i(block->rect.left - 2, block->rect.bottom - 2);
-    glVertex2i(block->rect.left, block->rect.bottom);
-    glVertex2i(block->rect.left, block->rect.top);
-    glVertex2i(block->rect.left - 2, block->rect.top);
-    glEnd();
-#endif
-#endif
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3f(0.6F, 0.6F, 0.8F);
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	block->rect.left - 1, block->rect.bottom - 1,
-	block->rect.left, block->rect.bottom - 1,
-	block->rect.left - 1, block->rect.top,
-	block->rect.left, block->rect.top
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
+    if (orthoCGO){
+      CGOColor(orthoCGO, 0.6F, 0.6F, 0.8F);
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, block->rect.left - 1, block->rect.bottom - 1, 0.f);
+      CGOVertex(orthoCGO, block->rect.left, block->rect.bottom - 1, 0.f);
+      CGOVertex(orthoCGO, block->rect.left, block->rect.top, 0.f);
+      CGOVertex(orthoCGO, block->rect.left - 1, block->rect.top, 0.f);
+      CGOEnd(orthoCGO);
+    } else {
+      glColor3f(0.6F, 0.6F, 0.8F);
+      glBegin(GL_POLYGON);
+      glVertex2i(block->rect.left - 1, block->rect.bottom - 1);
+      glVertex2i(block->rect.left, block->rect.bottom - 1);
+      glVertex2i(block->rect.left, block->rect.top);
+      glVertex2i(block->rect.left - 1, block->rect.top);
+      glEnd();
     }
-#else
-    glBegin(GL_POLYGON);
-    glVertex2i(block->rect.left - 1, block->rect.bottom - 1);
-    glVertex2i(block->rect.left, block->rect.bottom - 1);
-    glVertex2i(block->rect.left, block->rect.top);
-    glVertex2i(block->rect.left - 1, block->rect.top);
-    glEnd();
-#endif
-#endif
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3fv(block->BackColor);
-#endif
-    BlockFill(block);
+    if (orthoCGO)
+      CGOColorv(orthoCGO, block->BackColor);
+    else
+      glColor3fv(block->BackColor);
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3fv(block->TextColor);
-#endif
+    BlockFill(block ORTHOCGOARGVAR);
+
+    if (orthoCGO)
+      CGOColorv(orthoCGO, block->TextColor);
+    else
+      glColor3fv(block->TextColor);
 
     if(I->Selected >= 0) {
 
@@ -805,99 +770,69 @@ void PopUpDraw(Block * block)
       y = I->Block->rect.top - PopUpConvertY(I, I->Selected, true) - cPopUpCharMargin;
 
       y += 2;
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	x, y,
-	x + I->Width - 1, y,
-	x, y - (cPopUpLineHeight + 3),
-	x + I->Width - 1, y - (cPopUpLineHeight + 3)
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
-    }
-#else
+    if (orthoCGO){
+      CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+      CGOVertex(orthoCGO, x, y, 0.f);
+      CGOVertex(orthoCGO, x + I->Width - 1, y, 0.f);
+      CGOVertex(orthoCGO, x, y - (cPopUpLineHeight + 3), 0.f);
+      CGOVertex(orthoCGO, x + I->Width - 1, y - (cPopUpLineHeight + 3), 0.f);
+      CGOEnd(orthoCGO);
+    } else {
       glBegin(GL_POLYGON);
       glVertex2i(x, y);
       glVertex2i(x + I->Width - 1, y);
       glVertex2i(x + I->Width - 1, y - (cPopUpLineHeight + 3));
       glVertex2i(x, y - (cPopUpLineHeight + 3));
       glEnd();
-#endif
-#endif
     }
-
+    }
     if(I->Code[0] == 2) {       /* menu name */
 
       if(!SettingGetGlobal_i(G, cSetting_internal_gui_mode)) {
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-        glColor3f(0.3F, 0.3F, 0.6F);
-#endif
+	if (orthoCGO)
+	  CGOColor(orthoCGO, 0.3F, 0.3F, 0.6F);
+	else
+	  glColor3f(0.3F, 0.3F, 0.6F);
       } else {
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-        glColor3f(1.0F, 1.0F, 1.0F);
-#endif
+	if (orthoCGO)
+	  CGOColor(orthoCGO, 1.0F, 1.0F, 1.0F);
+	else
+	  glColor3f(1.0F, 1.0F, 1.0F);
       }
       x = I->Block->rect.left;
       y = I->Block->rect.top;
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	x, y,
-	x + I->Width, y,
-	x, y - (cPopUpLineHeight + cPopUpCharMargin),
-	x + I->Width, y - (cPopUpLineHeight + cPopUpCharMargin)
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_VERTEX_ARRAY);
-    }
-#else
-      glBegin(GL_POLYGON);
-      glVertex2i(x, y);
-      glVertex2i(x + I->Width, y);
-      glVertex2i(x + I->Width, y - (cPopUpLineHeight + cPopUpCharMargin));
-      glVertex2i(x, y - (cPopUpLineHeight + cPopUpCharMargin));
-      glEnd();
-#endif
-#endif
+      if (orthoCGO){
+	CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+	CGOVertex(orthoCGO, x, y, 0.f);
+	CGOVertex(orthoCGO, x + I->Width, y, 0.f);
+	CGOVertex(orthoCGO, x, y - (cPopUpLineHeight + cPopUpCharMargin), 0.f);
+	CGOVertex(orthoCGO, x + I->Width, y - (cPopUpLineHeight + cPopUpCharMargin), 0.f);
+	CGOEnd(orthoCGO);
+      } else {
+	glBegin(GL_POLYGON);
+	glVertex2i(x, y);
+	glVertex2i(x + I->Width, y);
+	glVertex2i(x + I->Width, y - (cPopUpLineHeight + cPopUpCharMargin));
+	glVertex2i(x, y - (cPopUpLineHeight + cPopUpCharMargin));
+	glEnd();
+      }
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-      glColor3f(0.2F, 0.2F, 0.4F);
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	x + I->Width - 1, y - (cPopUpLineHeight + cPopUpCharMargin),
-	x, y - (cPopUpLineHeight + cPopUpCharMargin)
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glDrawArrays(GL_LINES, 0, 2);
-      glDisableClientState(GL_VERTEX_ARRAY);
-    }
-#else
-      glBegin(GL_LINES);
-      glVertex2i(x + I->Width - 1, y - (cPopUpLineHeight + cPopUpCharMargin));
-      glVertex2i(x, y - (cPopUpLineHeight + cPopUpCharMargin));
-      glEnd();
-#endif
-#endif
+      if (orthoCGO){
+	CGOColor(orthoCGO, 0.2F, 0.2F, 0.4F);
+	CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+	CGOVertex(orthoCGO, x + I->Width - 1, y - (cPopUpLineHeight + cPopUpCharMargin), 0.f);
+	CGOVertex(orthoCGO, x + I->Width - 1, y - (cPopUpLineHeight + cPopUpCharMargin) - 1.f, 0.f);
+	CGOVertex(orthoCGO, x, y - (cPopUpLineHeight + cPopUpCharMargin), 0.f);
+	CGOVertex(orthoCGO, x, y - (cPopUpLineHeight + cPopUpCharMargin) - 1.f, 0.f);
+	CGOEnd(orthoCGO);
+      } else {
+	glColor3f(0.2F, 0.2F, 0.4F);
+	glBegin(GL_LINES);
+	glVertex2i(x + I->Width - 1, y - (cPopUpLineHeight + cPopUpCharMargin));
+	glVertex2i(x, y - (cPopUpLineHeight + cPopUpCharMargin));
+	glEnd();
+      }
     }
 
     x = I->Block->rect.left + cPopUpCharMargin;
@@ -933,142 +868,107 @@ void PopUpDraw(Block * block)
                 }
 
           TextSetPos2i(G, xx, y + cPopUpCharLift);
-          TextDrawChar(G, *(c++));
+          TextDrawChar(G, *(c++) ORTHOCGOARGVAR);
           xx = xx + 8;
         }
 
         if(I->Sub[a]) {
 
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	I->Block->rect.left - 3, y + 1,
-	I->Block->rect.left, y + 1,
-	I->Block->rect.left - 3, y + ((cPopUpLineHeight)) - 4,
-	I->Block->rect.left, y + ((cPopUpLineHeight)) - 4
-      };
-      const GLfloat colorVerts[] = {
-	0.4F, 0.4F, 0.4F, 1.f,
-	0.1F, 0.1F, 0.1F, 1.f,
-	0.4F, 0.4F, 0.4F, 1.f,
-	0.1F, 0.1F, 0.1F, 1.f
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glEnableClientState(GL_COLOR_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glColorPointer(4, GL_FLOAT, 0, colorVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_COLOR_ARRAY);
-      glDisableClientState(GL_VERTEX_ARRAY);
-    }
-#else
-          glBegin(GL_POLYGON);
-          glColor3f(0.4F, 0.4F, 0.4F);
-          glVertex2i(I->Block->rect.left - 3, y + 1);
-          glColor3f(0.1F, 0.1F, 0.1F);
-          glVertex2i(I->Block->rect.left, y + 1);
-          glVertex2i(I->Block->rect.left, y + ((cPopUpLineHeight)) - 4);
-          glColor3f(0.4F, 0.4F, 0.4F);
-          glVertex2i(I->Block->rect.left - 3, y + ((cPopUpLineHeight)) - 4);
-          glEnd();
-#endif
-#endif
-
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-	I->Block->rect.right, y + 1,
-	I->Block->rect.right + 3, y + 1,
-	I->Block->rect.right, y + ((cPopUpLineHeight)) - 4,
-	I->Block->rect.right + 3, y + ((cPopUpLineHeight)) - 4
-      };
-      const GLfloat colorVerts[] = {
-	0.1F, 0.2F, 0.2F, 1.f,
-	0.4F, 0.4F, 0.4F, 1.f,
-	0.1F, 0.2F, 0.2F, 1.f,
-	0.4F, 0.4F, 0.4F, 1.f,
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glEnableClientState(GL_COLOR_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glColorPointer(4, GL_FLOAT, 0, colorVerts);
-      glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-      glDisableClientState(GL_COLOR_ARRAY);
-      glDisableClientState(GL_VERTEX_ARRAY);
-    }
-#else
-          glBegin(GL_POLYGON);
-          glColor3f(0.1F, 0.2F, 0.2F);
-          glVertex2i(I->Block->rect.right, y + 1);
-          glColor3f(0.4F, 0.4F, 0.4F);
-          glVertex2i(I->Block->rect.right + 3, y + 1);
-          glVertex2i(I->Block->rect.right + 3, y + ((cPopUpLineHeight)) - 4);
-          glColor3f(0.1F, 0.2F, 0.2F);
-          glVertex2i(I->Block->rect.right, y + ((cPopUpLineHeight)) - 4);
-          glEnd();
-#endif
-#endif
+	  if (orthoCGO){
+	    CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+	    CGOColor(orthoCGO, 0.4F, 0.4F, 0.4F);
+	    CGOVertex(orthoCGO, I->Block->rect.left - 3, y + ((cPopUpLineHeight)) - 4, 0.f);
+	    CGOColor(orthoCGO, 0.4F, 0.4F, 0.4F);
+	    CGOVertex(orthoCGO, I->Block->rect.left - 3, y + 1, 0.f);
+	    CGOColor(orthoCGO, 0.1F, 0.1F, 0.1F);
+	    CGOVertex(orthoCGO, I->Block->rect.left, y + ((cPopUpLineHeight)) - 4, 0.f);
+	    CGOColor(orthoCGO, 0.1F, 0.1F, 0.1F);
+	    CGOVertex(orthoCGO, I->Block->rect.left, y + 1, 0.f);
+	    CGOEnd(orthoCGO);
+	  } else {
+	    glBegin(GL_POLYGON);
+	    glColor3f(0.4F, 0.4F, 0.4F);
+	    glVertex2i(I->Block->rect.left - 3, y + 1);
+	    glColor3f(0.1F, 0.1F, 0.1F);
+	    glVertex2i(I->Block->rect.left, y + 1);
+	    glVertex2i(I->Block->rect.left, y + ((cPopUpLineHeight)) - 4);
+	    glColor3f(0.4F, 0.4F, 0.4F);
+	    glVertex2i(I->Block->rect.left - 3, y + ((cPopUpLineHeight)) - 4);
+	    glEnd();
+	  }
+	  if (orthoCGO){
+	    CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+	    CGOColor(orthoCGO, 0.1F, 0.2F, 0.2F);
+	    CGOVertex(orthoCGO, I->Block->rect.right, y + 1, 0.f);
+	    CGOColor(orthoCGO, 0.4F, 0.4F, 0.4F);
+	    CGOVertex(orthoCGO, I->Block->rect.right + 3, y + 1, 0.f);
+	    CGOColor(orthoCGO, 0.1F, 0.2F, 0.2F);
+	    CGOVertex(orthoCGO, I->Block->rect.right, y + ((cPopUpLineHeight)) - 4, 0.f);
+	    CGOColor(orthoCGO, 0.4F, 0.4F, 0.4F);
+	    CGOVertex(orthoCGO, I->Block->rect.right + 3, y + ((cPopUpLineHeight)) - 4, 0.f);
+	    CGOEnd(orthoCGO);
+	  } else {
+	    glBegin(GL_POLYGON);
+	    glColor3f(0.1F, 0.2F, 0.2F);
+	    glVertex2i(I->Block->rect.right, y + 1);
+	    glColor3f(0.4F, 0.4F, 0.4F);
+	    glVertex2i(I->Block->rect.right + 3, y + 1);
+	    glVertex2i(I->Block->rect.right + 3, y + ((cPopUpLineHeight)) - 4);
+	    glColor3f(0.1F, 0.2F, 0.2F);
+	    glVertex2i(I->Block->rect.right, y + ((cPopUpLineHeight)) - 4);
+	    glEnd();
+	  }
         }
 
         y -= cPopUpLineHeight;
         if(I->Code[a] == 2)
           y -= 2;
       } else {
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-#ifdef _PYMOL_GL_DRAWARRAYS
-    {
-      const GLint polyVerts[] = {
-        I->Block->rect.left, y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 3,
-        I->Block->rect.right, y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 3,
-        I->Block->rect.left, y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 4,
-        I->Block->rect.right, y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 4
-      };
-      const GLfloat colorVerts[] = {
-        0.3F, 0.3F, 0.5F, 1.f,
-        0.3F, 0.3F, 0.5F, 1.f,
-        0.6F, 0.6F, 0.8F, 1.f,
-        0.6F, 0.6F, 0.8F, 1.f
-      };
-      glEnableClientState(GL_VERTEX_ARRAY);
-      glEnableClientState(GL_COLOR_ARRAY);
-      glVertexPointer(2, GL_INT, 0, polyVerts);
-      glColorPointer(4, GL_FLOAT, 0, colorVerts);
-      glDrawArrays(GL_LINES, 0, 4);
-      glDisableClientState(GL_COLOR_ARRAY);
-      glDisableClientState(GL_VERTEX_ARRAY);
-    }
-#else
-        glBegin(GL_LINES);
-        glColor3f(0.3F, 0.3F, 0.5F);
-        glVertex2i(I->Block->rect.left,
-                   y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 3);
-        glVertex2i(I->Block->rect.right,
-                   y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 3);
-        glColor3f(0.6F, 0.6F, 0.8F);
-        glVertex2i(I->Block->rect.left,
-                   y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 4);
-        glVertex2i(I->Block->rect.right,
-                   y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 4);
-        glEnd();
-#endif
-#endif
+	  if (orthoCGO){
+	    /* two lines between sections in the menu, one light, one dark */
+	    CGOColor(orthoCGO, 0.3F, 0.3F, 0.5F);
+	    CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+	    CGOVertex(orthoCGO, I->Block->rect.right,
+		      y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 4, 0.f);
+	    CGOVertex(orthoCGO, I->Block->rect.right,
+		      y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 3, 0.f);
+	    CGOVertex(orthoCGO, I->Block->rect.left,
+		      y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 4, 0.f);
+	    CGOVertex(orthoCGO, I->Block->rect.left,
+		      y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 3, 0.f);
+	    CGOEnd(orthoCGO);
+	    CGOColor(orthoCGO, 0.6F, 0.6F, 0.8F);
+	    CGOBegin(orthoCGO, GL_TRIANGLE_STRIP);
+	    CGOVertex(orthoCGO, I->Block->rect.right,
+		      y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 5, 0.f);
+	    CGOVertex(orthoCGO, I->Block->rect.right,
+		      y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 4, 0.f);
+	    CGOVertex(orthoCGO, I->Block->rect.left,
+		      y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 5, 0.f);
+	    CGOVertex(orthoCGO, I->Block->rect.left,
+		      y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 4, 0.f);
+	    CGOEnd(orthoCGO);
+	  } else {
+	    glBegin(GL_LINES);
+	    glColor3f(0.3F, 0.3F, 0.5F);
+	    glVertex2i(I->Block->rect.left,
+		       y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 3);
+	    glVertex2i(I->Block->rect.right,
+		       y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 3);
+	    glColor3f(0.6F, 0.6F, 0.8F);
+	    glVertex2i(I->Block->rect.left,
+		       y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 4);
+	    glVertex2i(I->Block->rect.right,
+		       y + ((cPopUpLineHeight + cPopUpCharMargin) / 2) + 4);
+	    glEnd();
+	  }
         y -= cPopUpBarHeight;
       }
-
     }
-#ifdef PURE_OPENGL_ES_2
-    /* TODO */
-#else
-    glColor3fv(block->TextColor);
-#endif
+    if (orthoCGO)
+      CGOColorv(orthoCGO, block->TextColor);
+    else
+      glColor3fv(block->TextColor);
     /*    BlockOutline(block); */
   }
 }
