@@ -685,8 +685,8 @@ DESCRIPTION
         if is_string(active):
             active=boolean_dict[boolean_sc.auto_err(active,'active')]
         active = int(active)
-        if len(mouse_ring):
-            bm = int(_cmd.get_setting(_self._COb,"button_mode"))
+        bm = _self.get_setting_int("button_mode")
+        if 0 <= bm < len(mouse_ring):
             mouse_mode = mouse_ring[bm]
             if active:
                 if mouse_mode[0:10]=='two_button':
@@ -710,6 +710,17 @@ DESCRIPTION
 
     "set_key" binds a specific python function to a key press.
 
+    New in PyMOL 1.6.1: second argument can also be a string in PyMOL
+    command syntax.
+
+USAGE
+
+    set_key key, command
+
+EXAMPLE
+
+    set_key F1, as cartoon, polymer; as sticks, organic
+
 PYMOL API (ONLY)
 
     cmd.set_key( string key, function fn, tuple arg=(), dict kw={})
@@ -738,9 +749,14 @@ KEYS WHICH CAN BE REDEFINED
 
 SEE ALSO
 
-    button
+    button, alias
         '''
         r = DEFAULT_ERROR
+        if isinstance(fn, basestring):
+            if arg or kw:
+                raise ValueError('arg and kw must be empty if fn is string')
+            arg = (fn,)
+            fn = cmd.do
         if key[0:5]=='CTRL-':
             pat=key[5:]
             if len(pat)>1: # ctrl-special key
