@@ -183,15 +183,13 @@ DESCRIPTION
     
 USAGE
 
-    map_generate name, object, reflection_file, amplitudes, phases, weights [,
+    map_generate name, reflection_file, amplitudes, phases, weights [,
         reso_low [, reso_high ]]
 
 ARGUMENTS
 
     name = string: name of the map object to create or modify
 	
-    obj = string: molecular object from which Fc is calculated.
-
     reflection_file = string: name of reflection file on disk; if None, then
                       PyMOL attempts to download the CIF file from the PDB.
                       Default = None; attempt to download from PDB.
@@ -222,7 +220,7 @@ NOTES
     New in PyMOL v1.4 for Mac and Linux.
     
     '''
-        # preprocess selection
+        quiet = int(quiet)
         r = DEFAULT_ERROR
         try:
             _self.lock(_self)
@@ -235,66 +233,47 @@ NOTES
             mtzFile = headering.MTZHeader(reflection_file)
                 
             # FORMAT: crystal/dataset/column
-            dataset = string.split(amplitudes,"/")
-            if len(dataset):
-                datasetName = dataset[1]
-                
-            for d in mtzFile.datasets.keys():
-                if mtzFile.datasets[d]["name"]==datasetName:
+            _, datasetName, ampColName = ('//' + amplitudes).rsplit('/', 2)
+
+            # if datasetName is empty, take any dataset that has ampColName
+            for dataset in mtzFile.datasets.values():
+                if (not datasetName or dataset["name"] == datasetName) and \
+                        ampColName in dataset["cols"]:
                     break
-            dataset = mtzFile.datasets[d]
+            else:
+                raise pymol.CmdException("no dataset found")
+
             cellX, cellY, cellZ = dataset['x'], dataset['y'], dataset['z']
             cellAlpha, cellBeta, cellGamma = dataset['alpha'], dataset['beta'], dataset['gamma']
             if reso_low==reso_high:
                 reso_low, reso_high = mtzFile.reso_min, mtzFile.reso_max
             space_group = mtzFile.space_group
 
-            #print "space_group = '%s'" % space_group
-            
-            # got data, now parse for driver
-            
-            origAmplitudes = amplitudes
-            amplitudes = string.split(amplitudes,'/')
-            if len(amplitudes):
-                amplitudes = amplitudes[-1]
-            else:
-                print " MapGenerate-Error: Improperly formatted amplitude name, '%s'.  Please specify it as" % origAmplitudes
-                print "  dataset_name/crystal_name/amplitude_column_name."
-                return None
+            phases = phases.rsplit('/', 1)[-1]
+            if not phases:
+                raise pymol.CmdException("phase name missing")
 
-            origPhases = phases
-            phases = string.split(phases,'/')
-            if len(phases):
-                phases=phases[-1]
-            else:
-                print " MapGenerate-Error: Improperly formatted phase name, '%s'.  Please specify it as" % origPhases
-                print "  dataset_name/crystal_name/phase_column_name."
-                return None
-
-            if weights!="None":
-                origWeights = weights
-                weights = string.split(weights,'/')
-                if len(weights):
-                    weights = weights[-1]
-                else:
-                    print " MapGenerate-Error: Improperly formatted weight name, '%s'.  Please specify it as" % origWeights
-                    print "  dataset_name/crystal_name/weight_column_name."
-                    return None
+            if weights and weights!="None":
+                weights = weights.rsplit('/', 1)[-1]
+                if not weights:
+                    raise pymol.CmdException("Improperly formatted weights name")
 
             tempFile = tempfile.NamedTemporaryFile(delete=False)
             tempFileName = tempFile.name
             tempFile.close()
 
-            r = _cmd.map_generate(_self._COb,str(name),str(reflection_file),str(tempFileName),str(amplitudes),str(phases),
+            r = _cmd.map_generate(_self._COb,str(name),str(reflection_file),str(tempFileName),
+                              str(ampColName),str(phases),
                               str(weights),float(reso_low),float(reso_high),str(space_group),
                               float(cellX), float(cellY), float(cellZ),
                               float(cellAlpha), float(cellBeta), float(cellGamma),
                               int(quiet),int(zoom))
             if r!=None:
-                print "Loading map '%s'" % (name)
-                pymol.cmd.load(r, name, format="ccp4", finish=1)
-                # setting?
-                os.remove(r)
+                if not quiet:
+                    print "Loading map '%s'" % (name)
+                r = _self.load(r, name, format="ccp4", finish=1)
+
+            os.remove(tempFileName)
 
         except ImportError:
             print " MapGenerate-Error: Cannot import headering module.  Cannot read MTZ file or make map."
@@ -478,8 +457,12 @@ SEE ALSO
         if selection!='':
             selection = selector.process(selection)
         # coerce range
-        range = list(safe_list_eval(str(range)))
-        range = map(lambda x:float(x),range)
+        try:
+            if isinstance(range, str):
+                range = safe_list_eval(range)
+            range = map(float, range)
+        except:
+            raise pymol.CmdException('invalid range')
         if is_list(color):
             for a in color:
                 if not is_list(a):
@@ -1016,7 +999,7 @@ USAGE
                     reduce(operator.__add__,map(lambda a:a.coord[0],model.atom)),
                     reduce(operator.__add__,map(lambda a:a.coord[1],model.atom)),
                     reduce(operator.__add__,map(lambda a:a.coord[2],model.atom))])
-            r = _self.load_model(model,str(object),quiet=quiet,zoom=zoom)
+            r = _self.load_model(model,str(object),quiet=quiet,zoom=zoom, _self=_self)
         except IOError:
             print "Error: unable to load fragment '%s'." % name
         except:
@@ -1046,7 +1029,7 @@ ARGUMENTS
 
     source_state = integer: {default: 0 -- copy all states}
 
-    target_state = integer: {default: 0}
+    target_state = integer: -1 appends after last state {default: 0}
 
 PYMOL API
 
@@ -1064,22 +1047,16 @@ SEE ALSO
     load, copy, extract
         '''
         r = DEFAULT_ERROR
+        target_state = int(target_state)
+        if target_state == -1:
+            target_state = _self.count_states('?' + name) + 1
         # preprocess selection
         selection = selector.process(selection)
         #      
         try:
             _self.lock(_self)
             if name==None:
-                avoid = {}
-                for obj in cmd.get_names("all"):
-                    avoid[obj] = 1
-                sel_cnt = _cmd.get(_self._COb,"sel_counter") 
-                while 1:
-                    sel_cnt = sel_cnt + 1.0
-                    name = "obj%02.0f" % sel_cnt
-                    if not avoid.has_key(name):
-                        _cmd.legacy_set(_self._COb,"sel_counter","%1.0f" % sel_cnt)
-                        break
+                name = _self.get_unused_name("obj")
             r = _cmd.create(_self._COb,str(name),"("+str(selection)+")",
                             int(source_state)-1,int(target_state)-1,
                             int(discrete),int(zoom),int(quiet),int(singletons))
