@@ -170,21 +170,24 @@ PYMOL API
     
     def get_fastastr(selection="all", state=-1, quiet=1, _self=cmd):
         dict = { 'seq' : {} }
-        _self.iterate("("+selection+") and polymer and name ca",
+        # we use (alt '' or alt 'A') because 'guide' picks up 
+        # non-canonical structures: eg, 1ejg has residue 22 as a SER and 
+        # PRO, which guide will report twice
+        _self.iterate("("+selection+") and polymer and name ca and (alt '' or alt 'A')",
                     "seq[model]=seq.get(model,[]);seq[model].append(resn)",space=dict)
         seq = dict['seq']
         result = []
         for obj in _self.get_names("objects",selection='('+selection+')'):
             if seq.has_key(obj):
-                seq = map(lambda x:_resn_to_aa.get(x,'?'),seq[obj])
+                cur_seq = map(lambda x:_resn_to_aa.get(x,'?'),seq[obj])
                 result.append(">%s"%obj)
-                seq = string.join(seq,'')
-                while len(seq):
-                    if len(seq)>=70:
-                        result.append(seq[0:70])
-                        seq=seq[70:]
+                cur_seq = string.join(cur_seq,'')
+                while len(cur_seq):
+                    if len(cur_seq)>=70:
+                        result.append(cur_seq[0:70])
+                        cur_seq=cur_seq[70:]
                     else:
-                        result.append(seq)
+                        result.append(cur_seq)
                         break
         result = string.join(result,'\n')
         if len(result):
@@ -282,6 +285,20 @@ NOTES
             raise QuietException                  
         return r
         
+    def _unit2px(value, dpi):
+        '''API only. Returns pixel units given a string representation in other units'''
+        if isinstance(value, str):
+            m = re.search(r'[a-z].*', value, re.I)
+            if m:
+                if dpi <= 0:
+                    raise pymol.CmdException('need dpi if units are given')
+                value, unit = value[:m.start()], m.group(0).lower()
+                upi = {'in': 1.0, 'mm': 25.4, 'cm': 2.54}
+                if unit not in upi:
+                    raise pymol.CmdException('unknown unit, supported units are: ' +
+                            ', '.join(upi))
+                value = float(value) * dpi / upi[unit] + 0.5
+        return int(value)
 
     def png(filename, width=0, height=0, dpi=-1.0, ray=0,
             quiet=1, prior=0, format=0, _self=cmd):
@@ -298,9 +315,12 @@ ARGUMENTS
 
     filename = string: file path to be written
     
-    width = integer: width in pixels {default: 0 (current)}
+    width = integer or string: width in pixels (without units), inches (in)
+    or centimeters (cm). If unit suffix is given, dpi argument is required
+    as well. If only one of width or height is given, the aspect ratio of
+    the viewport is preserved. {default: 0 (current)}
 
-    height = integer: height in pixels {default: 0 (current)}
+    height = integer or string: height (see width) {default: 0 (current)}
 
     dpi = float: dots-per-inch {default -1.0 (unspecified)}
 
@@ -310,6 +330,7 @@ EXAMPLES
 
     png image.png
     png image.png, dpi=300
+    png image.png, 10cm, dpi=300, ray=1
 
 NOTES
 
@@ -334,6 +355,12 @@ PYMOL API
                 if prior < 0: # default is to fall back to actual rendering
                     prior = 0
         if not prior:
+            dpi = float(dpi)
+            if dpi < 0:
+                dpi = cmd.get_setting_float('image_dots_per_inch')
+            width = _unit2px(width, dpi)
+            height = _unit2px(height, dpi)
+
             if thread.get_ident() == pymol.glutThread:
                 r = _self._png(str(filename),int(width),int(height),float(dpi),
                                int(ray),int(quiet),0,int(format),_self)
@@ -586,20 +613,15 @@ SEE ALSO
                 print " Save: wrote \""+filename+"\"."
         elif format=='sdf':
             state = int(state)
-            if state==0:
-                first_state = 1
-                last_state = cmd.count_states(selection)
-            else:
-                first_state = state
-                last_state = state
             sdf = SDF(filename,'w')
-            for state in range(first_state, last_state+1):
+            for i, (selection, state) in enumerate(
+                    pymol.selecting.objsele_state_iter(selection, state)):
                 rec = SDFRec(io.mol.toList(_self.get_model(selection,state,ref,ref_state))
                              + ["$$$$\n"])
                 sdf.write(rec)
             r = DEFAULT_SUCCESS
             if not quiet:
-                print " Save: wrote %d states to \"%s\"."%(1+last_state-first_state,filename)
+                print " Save: wrote %d states to \"%s\"." % (i + 1, filename)
         elif format=='mol':
             io.mol.toFile(_self.get_model(selection,state,ref,ref_state),filename)
             r = DEFAULT_SUCCESS
@@ -607,22 +629,18 @@ SEE ALSO
                 print " Save: wrote \""+filename+"\"."
         elif format=="mol2":
             state = int(state)
-            if state==0:
-                first_state = 1
-                last_state = cmd.count_states(selection)
-            else:
-                first_state = state
-                last_state = state
-            recList=[]
-            for state in range(first_state, last_state+1):
-                # assign_atom_types selection, format [ mol2, macromodel ], state, quiet
-                assign_atom_types(selection, "mol2", state, 1, _self)
-                recList.extend(io.mol2.toList(_self.get_model(selection,state,ref,ref_state),selection=selection,state=state))
+            recList = []
+            entrycount = 0
+            for osele, ostate in pymol.selecting.objsele_state_iter(selection, state):
+                    assign_atom_types(osele, "mol2", ostate, 1, _self)
+                    recList.extend(io.mol2.toList(_self.get_model(osele,
+                        ostate, ref, ref_state), selection=osele, state=ostate))
+                    entrycount += 1
             m = MOL2(cmd=cmd)
             m.strToFile(recList,filename)
             r = DEFAULT_SUCCESS
             if not quiet:
-                print " Save: wrote %d states to \"%s\"."%(1+last_state-first_state,filename)
+                print ' Save: wrote %d entries to "%s".' % (entrycount, filename)
         elif format=='png':
             r = _self.png(filename,quiet=quiet)
         # refactor below to lift repeated code
