@@ -26,7 +26,8 @@ if __name__=='pymol.fitting':
 
 
         def cealign(target, mobile, target_state=1, mobile_state=1, quiet=1,
-                    guide=1, d0=3.0, d1=4.0, window=8, gap_max=30, transform=1, _self=cmd):
+                    guide=1, d0=3.0, d1=4.0, window=8, gap_max=30, transform=1,
+                    object=None, _self=cmd):
                 '''
 DESCRIPTION
 
@@ -60,11 +61,9 @@ SEE ALSO
     align, pair_fit, fit, rms, rms_cur, intra_rms, intra_rms_cur, super
                 '''
                 quiet = int(quiet)
+                window = int(window)
                 guide = "" if int(guide)==0 else "and guide"
 
-                # make the lists for holding coordinates
-                sel1, sel2 = [], []
-
                 mobile = "(%s) %s" % (mobile,guide)
                 target = "(%s) %s" % (target,guide)
 
@@ -72,21 +71,26 @@ SEE ALSO
                 mobile = selector.process(mobile)
                 target = selector.process(target)
                                 
-                _self.iterate_state(mobile_state, mobile, "sel.append([x,y,z])", space={'sel':sel2})
-                _self.iterate_state(target_state, target, "sel.append([x,y,z])", space={'sel':sel1})
+                # make the lists for holding coordinates and IDs
+                mod1 = _self.get_model(target, state=target_state)
+                mod2 = _self.get_model(mobile, state=mobile_state)
+                sel1 = mod1.get_coord_list()
+                sel2 = mod2.get_coord_list()
+                ids1 = [a.id for a in mod1.atom]
+                ids2 = [a.id for a in mod2.atom]
                 
-                if len(sel1)==0:
-                        print "CEalign-Error: Your first selection was empty."
-                        return None
-                if len(sel2)==0:
-                        print "CEalign-Error: Your second selection was empty."
-                        return None
-                if int(window) < 2:
+                if len(sel1) < 2 * window:
+                        print "CEalign-Error: Your target selection is too short."
+                        raise pymol.CmdException
+                if len(sel2) < 2 * window:
+                        print "CEalign-Error: Your mobile selection is too short."
+                        raise pymol.CmdException
+                if window < 3:
                         print "CEalign-Error: window size must be an integer greater than 2."
-                        return None
+                        raise pymol.CmdException
                 if int(gap_max) < 0:
                         print "CEalign-Error: gap_max must be a positive integer."
-                        return None
+                        raise pymol.CmdException
 
                 r = DEFAULT_ERROR
                 
@@ -96,7 +100,7 @@ SEE ALSO
                         # call the C function
                         r = _cmd.cealign( _self._COb, sel1, sel2, float(d0), float(d1), int(window), int(gap_max) )
 
-                        (aliLen, RMSD, rotMat) = r
+                        (aliLen, RMSD, rotMat, i1, i2) = r
                         if quiet==-1:
                                 import pprint
                                 print "RMSD %f over %i residues" % (float(RMSD), int(aliLen))
@@ -108,13 +112,31 @@ SEE ALSO
 			if int(transform):
 				for model in cmd.get_object_list("(" + mobile + ")"):
 					_self.transform_object(model, rotMat, state=0)
+
+                        if object is not None:
+                            obj1 = cmd.get_object_list("(" + target + ")")
+                            obj2 = cmd.get_object_list("(" + mobile + ")")
+                            if len(obj1) > 1 or len(obj2) > 1:
+                                print ' CEalign-Error: selection spans multiple' + \
+                                        ' objects, cannot create alignment object'
+                                raise pymol.CmdException
+                            tmp1 = _self.get_unused_name('_1')
+                            tmp2 = _self.get_unused_name('_2')
+                            _self.select_list(tmp1, obj1[0], [ids1[j] for i in i1 for j in range(i, i+window)])
+                            _self.select_list(tmp2, obj2[0], [ids2[j] for i in i2 for j in range(i, i+window)])
+                            _self.rms_cur(tmp2, tmp1, cycles=0, matchmaker=4, object=object)
+                            _self.delete(tmp1)
+                            _self.delete(tmp2)
+                except SystemError:
+                    # findBest might return NULL, which raises SystemError
+                    print " CEalign-Error: alignment failed"
                 finally:
                         _self.unlock(r,_self)
                 if _self._raising(r,_self): raise pymol.CmdException             
                 return ( {"alignment_length": aliLen, "RMSD" : RMSD, "rotation_matrix" : rotMat } )
 
 
-        def alignto(target,method="cealign",quiet=1,_self=cmd):
+        def alignto(target=None,method="cealign",quiet=1,_self=cmd, **kwargs):
                 """
 DESCRIPTION
 
@@ -136,22 +158,26 @@ EXAMPLE
 
         # align them to 1cll using cealign
         alignto 1cll, method=cealign
+        alignto 1cll, object=all_to_1cll
 
 SEE ALSO
 
         align, super, cealign, fit, rms, rms_cur, intra_fit
                 """
-                if method=="align":
-                        method=lambda x, sel : _self.align(x,sel)
-                elif method=="super":
-                        method=lambda x, sel : _self.super(x,sel)
-                else:
-                        method=lambda x, sel : _self.cealign(sel,x)
-                        
-                for x in cmd.get_names("public_objects"):
+                if cmd.is_string(method):
+                    if method in cmd.keyword:
+                        method = cmd.keyword[method][0]
+                    else:
+                        raise CmdException('Unknown method: ' + method)
+                names = cmd.get_names("public_objects", 1)
+                if not target:
+                    target = names[0]
+                for x in names:
+                        if x == target:
+                            continue
                         if not quiet:
                                 print "Aligning %s to %s" % (x, target)
-                        method(x,target)
+                        method(mobile=x, target=target, **kwargs)
                                 
 
 
@@ -196,12 +222,12 @@ SEE ALSO
                 target = selector.process(target)
                 if object==None: object=''
                 matrix = str(matrix)
-                if string.lower(matrix)=='none':
-                        matrix=''
-                if len(matrix):
-                        mfile = cmd.exp_path("$PYMOL_DATA/pymol/matrices/"+matrix)
+                if matrix.lower() in ['none', '']:
+                        mfile = ''
+                elif os.path.exists(matrix):
+                        mfile = matrix
                 else:
-                        mfile = ''                
+                        mfile = cmd.exp_path("$PYMOL_DATA/pymol/matrices/"+matrix)
                 # delete existing alignment object (if asked to reset it)
                 try:
                         _self.lock(_self)
@@ -263,12 +289,12 @@ SEE ALSO
                 mobile = selector.process(mobile)
                 target = selector.process(target)
                 matrix = str(matrix)
-                if string.lower(matrix)=='none':
-                        matrix=''
-                if len(matrix):
-                        mfile = cmd.exp_path("$PYMOL_DATA/pymol/matrices/"+matrix)
-                else:
+                if matrix.lower() in ['none', '']:
                         mfile = ''
+                elif os.path.exists(matrix):
+                        mfile = matrix
+                else:
+                        mfile = cmd.exp_path("$PYMOL_DATA/pymol/matrices/"+matrix)
                 if object==None: object=''
                 # delete existing alignment object (if asked to reset it)
                 try:
