@@ -95,12 +95,14 @@ def get_command_keywords(self_cmd=cmd):
         'get'           : [ self_cmd.get               , 0 , 0 , ''  , parsing.STRICT ],
         'get_angle'     : [ self_cmd.get_angle         , 0 , 0 , ''  , parsing.STRICT ],      
         'get_area'      : [ self_cmd.get_area          , 0 , 0 , ''  , parsing.STRICT ],
+        'get_bond'      : [ self_cmd.get_bond          , 0 , 0 , ''  , parsing.STRICT ],
         'get_chains'    : [ self_cmd.get_chains        , 0 , 0 , ''  , parsing.STRICT ],
         'get_dihedral'  : [ self_cmd.get_dihedral      , 0 , 0 , ''  , parsing.STRICT ],
         'get_distance'  : [ self_cmd.get_distance      , 0 , 0 , ''  , parsing.STRICT ],
         'get_extent'    : [ self_cmd.get_extent        , 0 , 0 , ''  , parsing.STRICT ],
         'get_position'  : [ self_cmd.get_position      , 0 , 0 , ''  , parsing.STRICT ],
         'get_symmetry'  : [ self_cmd.get_symmetry      , 0 , 0 , ''  , parsing.STRICT ],
+        'get_renderer'  : [ self_cmd.get_renderer      , 0 , 0 , ''  , parsing.STRICT ],
         'get_title'     : [ self_cmd.get_title         , 0 , 0 , ''  , parsing.STRICT ],   
         'get_type'      : [ self_cmd.get_type          , 0 , 0 , ''  , parsing.STRICT ],
         'get_version'   : [ self_cmd.get_version       , 0 , 0 , ''  , parsing.STRICT ],            
@@ -160,13 +162,13 @@ def get_command_keywords(self_cmd=cmd):
         'minsert'       : [ self_cmd.minsert           , 0 , 0 , ''  , parsing.STRICT ],
         'mmove'         : [ self_cmd.mmove             , 0 , 0 , ''  , parsing.STRICT ],        
         'move'          : [ self_cmd.move              , 0 , 0 , ''  , parsing.STRICT ],
+        'mse2met'       : [ self_cmd.mse2met           , 0 , 0 , ''  , parsing.STRICT ],
         'mset'          : [ self_cmd.mset              , 0 , 0 , ''  , parsing.STRICT ],
         'mdo'           : [ self_cmd.mdo               , 2 , 2 , ':' , parsing.MOVIE  ],
         'mdump'         : [ self_cmd.mdump             , 0 , 0 , ''  , parsing.STRICT ],      
         'mpng'          : [ self_cmd.mpng              , 0 , 0 , ''  , parsing.SECURE ],
         'mplay'         : [ self_cmd.mplay             , 0 , 0 , ''  , parsing.STRICT ],
         'mtoggle'       : [ self_cmd.mtoggle           , 0 , 0 , ''  , parsing.STRICT ],         
-        'mray'          : [ self_cmd.mray              , 0 , 0 , ''  , parsing.STRICT ],
         'mstop'         : [ self_cmd.mstop             , 0 , 0 , ''  , parsing.STRICT ],
         'mclear'        : [ self_cmd.mclear            , 0 , 0 , ''  , parsing.STRICT ],
         'middle'        : [ self_cmd.middle            , 0 , 0 , ''  , parsing.STRICT ],
@@ -231,7 +233,7 @@ def get_command_keywords(self_cmd=cmd):
         'set_geometry'  : [ self_cmd.set_geometry      , 0 , 0 , ''  , parsing.STRICT ],
         'set_symmetry'  : [ self_cmd.set_symmetry      , 0 , 0 , ''  , parsing.STRICT ],         
         'set_title'     : [ self_cmd.set_title         , 0 , 0 , ''  , parsing.STRICT ],   
-        'set_key'       : [ self_cmd.set_key           , 0 , 0 , ''  , parsing.STRICT ], # API only
+        'set_key'       : [ self_cmd.set_key           , 0 , 0 , ''  , parsing.LITERAL1 ], # insecure
         'set_view'      : [ self_cmd.set_view          , 0 , 0 , ''  , parsing.STRICT ],   
         'show'          : [ self_cmd.show              , 0 , 0 , ''  , parsing.STRICT ],
         'slice_new'     : [ self_cmd.slice_new         , 0 , 0 , ''  , parsing.STRICT ],
@@ -242,6 +244,7 @@ def get_command_keywords(self_cmd=cmd):
         'spawn'         : [ self_cmd.helping.spawn     , 1 , 2 , ',' , parsing.SPAWN  ], # insecure
         'spheroid'      : [ self_cmd.spheroid          , 0 , 0 , ''  , parsing.STRICT ],
         'splash'        : [ self_cmd.splash            , 0 , 0 , ''  , parsing.STRICT ],
+        'split_chains'  : [ self_cmd.split_chains      , 0 , 0 , ''  , parsing.STRICT ],
         'split_states'  : [ self_cmd.split_states      , 0 , 0 , ''  , parsing.STRICT ],
         '_special'      : [ self_cmd._special          , 0 , 0 , ''  , parsing.STRICT ],
         'stereo'        : [ self_cmd.stereo            , 0 , 0 , ''  , parsing.STRICT ],
